package com.monkeycoder.monkeynes.base;

import java.io.*;
import java.util.*;

public class DebugLog
{
  public static final int MODE_NULL       = 0;    // Debug output is ignored
  public static final int MODE_STD_OUT    = 1;    // Debug is directed to System.out
  public static final int MODE_STD_ERR    = 2;    // Debug is directed to System.err
  public static final int MODE_FILE       = 3;    // Debug is appended to the file "debug.log"
  
  private static DebugLog   inst          = null;
  private        int        mode          = MODE_NULL;
  private        int        prevMode      = MODE_NULL;
  private        int        vThresh       = 0;  // only things with this verbosity or lower display (defaults to nothing showing up)
  private        int        prevVThresh   = 0;
  private        FileWriter file          = null;
  
  private DebugLog()
  {
    try
    {
      file = new FileWriter( "debug.log", true );
      file.write( "=================================================\r\n" );
      file.write( "Log Started: " + new Date() + "\r\n" );
      file.write( "=================================================\r\n" );
      file.flush();
    }
    catch( IOException ioe )
    {}
  }
  
  public static DebugLog getInst()
  {
    if( inst == null )
      inst = new DebugLog();
    
    return inst;  
  }
  
  private void iSetMode( int inMode )
  {
    prevMode = mode;
    mode     = inMode;
  }
  
  private int iGetMode()
  {
    return mode;
  }
  
  private void iRestoreLastMode()
  {
    mode = prevMode;
  }  
  
  
  private void iSetVerbosity( int inVThresh )
  {
    prevVThresh = vThresh;
    vThresh     = inVThresh;
  }
  
  private int iGetVerbosity()
  {
    return vThresh;
  }
  
  private void iRestoreLastVerbosity()
  {
    vThresh = prevVThresh;
  }   
  
  // Internal logger
  private void iLog( int inVerbosity, String inText )
  {
    try
    {
      if( inVerbosity <= vThresh )
      {
        switch( mode )
        {
          case MODE_NULL:
            break;
          case MODE_STD_OUT:
            System.out.println( inText );
            break;
          case MODE_STD_ERR:
            System.err.println( inText );
            break;  
          case MODE_FILE:
            file.write( inText + "\r\n" );
            file.flush();
            break;  
        }
      }  
    }
    catch( IOException ioe )
    {}
  }
  
  // The publicly accessible static functions --------------------------
  
  // defaults to max 
  public static void log( String inText )
  {
    DebugLog.getInst().iLog( 5, inText );
  }
  
  public static void log( int inVerbosity, String inText )
  {
    DebugLog.getInst().iLog( inVerbosity, inText );
  }  
  
  public static void setMode( int inMode )
  {
    DebugLog.getInst().iSetMode( inMode );
  }
  
  public static int getMode()
  {
    return DebugLog.getInst().iGetMode();
  }
  
  public static void restoreLastMode()
  {
    DebugLog.getInst().iRestoreLastMode();
  }  
  
  public static void setVerbosity( int inVThresh )
  {
    DebugLog.getInst().iSetVerbosity( inVThresh );
  }
  
  public static int getVerbosity()
  {
    return DebugLog.getInst().iGetVerbosity();
  }
  
  public static void restoreLastVerbosity()
  {
    DebugLog.getInst().iRestoreLastVerbosity();
  }  
  
}