package com.monkeycoder.monkeynes.gui;

import com.monkeycoder.monkeynes.base.*;
import com.monkeycoder.monkeynes.hardware.*;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;


public class OpCodeTestShellGui implements Observer
{
  private Frame              parentFrame                      = null;
  private JDialog            mainDialog                       = null;
  private boolean            visible                          = false;

  private NesHardware        nes                              = null;

  private Document    outputDoc       = new DefaultStyledDocument();
  private JTextArea   outputArea      = new JTextArea( outputDoc, "", 15, 30 );
  private JScrollPane outputScroll    = new JScrollPane( outputArea );

  private JPanel      bottomPanel     = new JPanel( new BorderLayout() );
  private JTextField  inputText       = new JTextField( "", 200 );
  
  private ArrayList   cmdHistory      = new ArrayList( 10 );
  private int         histIndex       = -1;

  public OpCodeTestShellGui( Frame inParentFrame, NesHardware inHardware )
  {
    parentFrame = inParentFrame;
    nes         = inHardware;

    //mainMem.addObserver( this );

    // output area
    outputArea.setFont( new Font( "Courier", Font.PLAIN, 12 ) );
    outputArea.setEditable( false );

    // bottom panel
    bottomPanel.add( new JLabel( ">" ), BorderLayout.WEST   );
    bottomPanel.add( inputText,         BorderLayout.CENTER );

    // Dialog
    mainDialog = new JDialog( parentFrame, "OpCode Test Shell", false );

    // Build the main view
    mainDialog.getContentPane().add( outputScroll,    BorderLayout.CENTER  );
    mainDialog.getContentPane().add( bottomPanel,     BorderLayout.SOUTH  );


    // Input field listener (for pressing enter)
    inputText.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        String command = inputText.getText();
        inputText.setText( "" );

        commandProcessor( command );
        cmdHistory.add( new String( command ) );
        histIndex = -1;

        outputArea.setCaretPosition( outputArea.getText().length() );
        inputText.requestFocus();
      }
    } );
    
    // Input field listener (for pressing enter)
    inputText.addKeyListener( new KeyAdapter()
    {
      public void keyPressed( KeyEvent ke )
      {
        boolean controlKey = false;
        
        // Up Arrow
        if( ke.getKeyCode() == 38 )
        {
          if( histIndex == -1 )
            histIndex = cmdHistory.size() - 1;
          else if( histIndex > 0 )
            histIndex--;
          
          controlKey = true;  
        }
        // Down Arrow
        else if( ke.getKeyCode() == 40 )
        {
          if( histIndex != -1 && histIndex < (cmdHistory.size() - 1) )
            histIndex++;
          else
            histIndex = -1;  
            
          controlKey = true;  
        }        
        
        if( controlKey )
        {
          if( histIndex == -1 )
            inputText.setText( "" );
          else
            inputText.setText( (String)cmdHistory.get( histIndex ) );
        }    
      }
    } );

    // Main window closing listener
    mainDialog.addWindowListener( new WindowAdapter()
    {
      public void windowClosing( WindowEvent we )
      {
        mainDialog.hide();
        visible = false;
      }
    } );
  } // End of constructor ----------------------------------------------------------


  public void update( Observable o, Object arg )
  {
    if( visible )
    {
      // Do nothing for now...
    }
  }

  private void commandProcessor( String inCommand )
  {
    StringTokenizer cmdTok  = new StringTokenizer( inCommand );
    String          keyTok  = "";
    String          subTok1 = "";
    String          subTok2 = "";
    int             cycles  = 0;

    if( cmdTok.hasMoreTokens() )
    {
      outputArea.append( "> " + inCommand + "\n" );

      keyTok = cmdTok.nextToken();

      // ------------------------------------------------------------------------------------------
      if( keyTok.equalsIgnoreCase( "help" ) )
      {
        if( cmdTok.hasMoreTokens() )
        {
          subTok1 = cmdTok.nextToken();

          if( subTok1.equalsIgnoreCase( "clear" ) )
          {
            outputArea.append( "Syntax:  clear\n" );
            outputArea.append( "  Clears the output buffer.\n" );
          }
          else if( subTok1.equalsIgnoreCase( "#" ) )
          {
            outputArea.append( "Syntax:  # [comment]\n" );
            outputArea.append( "  comment - Text that will not be parsed by the command processor.\n" );
            outputArea.append( "            This text will still be outputted to the shell.\n" );
            outputArea.append( "- Note that the space following the # is required as the # is \n" );
            outputArea.append( "  actually the command.\n" );

          }
          else if( subTok1.equalsIgnoreCase( "help" ) )
          {
            outputArea.append( "Syntax:  help [command]\n" );
            outputArea.append( "  command - The command for which you want more detailed help.\n" );
          }
          else if( subTok1.equalsIgnoreCase( "load" ) )
          {
            outputArea.append( "Syntax:  load [script]\n" );
            outputArea.append( "  script - The path to a script file containing a sequence of\n" );
            outputArea.append( "           test commands to execute.\n" );
          }
          else if( subTok1.equalsIgnoreCase( "memget" ) )
          {
            outputArea.append( "Syntax:  memget [value]\n" );
            outputArea.append( "  addr  - 2 byte hex position in memory to get the value from.\n" );
            outputArea.append( "- Accesses main system memory.\n" );
          }
          else if( subTok1.equalsIgnoreCase( "memset" ) )
          {
            outputArea.append( "Syntax:  memset [addr] [value]\n" );
            outputArea.append( "  addr  - 2 byte hex position in memory to set.\n" );
            outputArea.append( "  value - 1 byte hex value to set.\n" );
            outputArea.append( "- Accesses main system memory.\n" );
          }
          else if( subTok1.equalsIgnoreCase( "op" ) )
          {
            outputArea.append( "Syntax:  op [opcode] [param]\n" );
            outputArea.append( "  opcode - The hex of the opcode to execute.\n" );
            outputArea.append( "  param  - The 0 to 2 bytes of hex to use as the params to the\n" );
            outputArea.append( "           opcode.\n" );
            outputArea.append( "- For more info about opcodes and their expected params please\n" );
            outputArea.append( "  check out http://monkeycoder.com/monkeynes/opcodes.php\n" );
            outputArea.append( "- All hex is expected as plain text with no prefix of '$' or '0x'\n" );
            outputArea.append( "  and is expected to be 0 padded appropriately.\n" );
            outputArea.append( "- Please note param values are expected low order byte first like\n" );
            outputArea.append( "  they would be from the rom.\n" );
          }
          else if( subTok1.equalsIgnoreCase( "opx" ) )
          {
            outputArea.append( "Syntax:  opx [6502]\n" );
            outputArea.append( "  6502 - Operation to execute in 6502 assembly.\n" );
            outputArea.append( "- For more info about opcode syntax please check out\n" );
            outputArea.append( "  http://monkeycoder.com/monkeynes/opcodes.php\n" );
          }
          else if( subTok1.equalsIgnoreCase( "power" ) )
          {
            outputArea.append( "Syntax:  power [state]\n" );
            outputArea.append( "  state - on|off - turns on or off the NES hardware.\n" );
            outputArea.append( "                   (just sets register values)\n" );
          }
          else if( subTok1.equalsIgnoreCase( "regs" ) )
          {
            outputArea.append( "Syntax:  regs\n" );
            outputArea.append( "  This will output the current state of all the registers in\n" );
            outputArea.append( "  the CPU.  Very similar to the CPU State Viewer output except\n" );
            outputArea.append( "  that it's in text.\n" );
          }
          else if( subTok1.equalsIgnoreCase( "save" ) )
          {
            outputArea.append( "Syntax:  save [file]\n" );
            outputArea.append( "  file - The path to a file to which the shell output will\n" );
            outputArea.append( "         be saved.\n" );
          }
          else if( subTok1.equalsIgnoreCase( "vidget" ) )
          {
            outputArea.append( "Syntax:  vidget [value]\n" );
            outputArea.append( "  addr  - 2 byte hex position in memory to get the value from.\n" );
            outputArea.append( "- Accesses video memory.\n" );
          }
          else if( subTok1.equalsIgnoreCase( "vidset" ) )
          {
            outputArea.append( "Syntax:  vidget [addr] [value]\n" );
            outputArea.append( "  addr  - 2 byte hex position in memory to set.\n" );
            outputArea.append( "  value - 1 byte hex value to set.\n" );
            outputArea.append( "- Accesses video memory.\n" );
          }
          else if( subTok1.equalsIgnoreCase( "monkey" ) )
          {
            outputArea.append( "Help monkey?  Aren't I already?\n" );
          }
          else
          {
            outputArea.append( "No help available for command: '" + subTok1 + "'\n" );
          }
        }
        else
        {
          outputArea.append( "help [command] - for more detailed help on a specific command.\n\n" );
          outputArea.append( "Command List:\n" );
          outputArea.append( "  #         clear     help      load      memget    \n" );
          outputArea.append( "  memset    op        opx       power     regs      \n" );
          outputArea.append( "  save      vidget    vidset    \n" );
        }
      }
      // ------------------------------------------------------------------------------------------
      else if( keyTok.equalsIgnoreCase( "#" ) )
      {
        // The output for this command already displayed on the prompt line.
      }
      // ------------------------------------------------------------------------------------------
      else if( keyTok.equalsIgnoreCase( "clear" ) )
      {
        initialize();
      }
      // ------------------------------------------------------------------------------------------
      else if( keyTok.equalsIgnoreCase( "load" ) )
      {
        if( cmdTok.hasMoreTokens() )
        {
          subTok1 = inCommand.substring( 5 );

          try
          {
            LineNumberReader lnr = new LineNumberReader( new FileReader( subTok1 ) );
            String           cur = "";

            outputArea.append( "Executing Script: '" + subTok1 + "'.\n\n" );
            while( (cur = lnr.readLine()) != null )
            {
              commandProcessor( cur );
              outputArea.setCaretPosition( outputArea.getText().length() );
            }
            outputArea.append( "Script Complete.\n" );

            lnr.close();
          }
          catch( IOException ioe )
          {
            outputArea.append( "Failed to Access Script: '" + subTok1 + "'.\n\n" );
            return;
          }
        }
        else
        {
          outputArea.append( "Expected Script File Name.\n" );
        }
      }
      // ------------------------------------------------------------------------------------------
      else if( keyTok.equalsIgnoreCase( "memget" ) )
      {
        int param1 = 0;

        // Load the params
        subTok1 = null;
        if( cmdTok.hasMoreTokens() )
          subTok1 = cmdTok.nextToken();
        try
        {
          param1 = Integer.parseInt( subTok1, 16 );
        }
        catch( NumberFormatException nfe )
        {
          outputArea.append( "Address must be numeric.\n\n" );
          return;
        }

        String hexVal = Integer.toString( nes.getMainMemory().readNoMessage( param1 ), 16 );
        if( hexVal.length() == 1 )
          hexVal = "0" + hexVal;
        outputArea.append( "Main Memory[$" + subTok1.toUpperCase() + "]:  $" + hexVal.toUpperCase() + "\n" );
      }
      // ------------------------------------------------------------------------------------------
      else if( keyTok.equalsIgnoreCase( "memset" ) )
      {
        int param1 = 0;
        int param2 = 0;

        // Load the params
        subTok1 = null;
        subTok2 = null;
        if( cmdTok.hasMoreTokens() )
          subTok1 = cmdTok.nextToken();
        if( cmdTok.hasMoreTokens() )
          subTok2 = cmdTok.nextToken();
        try
        {
          param1 = Integer.parseInt( subTok1, 16 );
          param2 = Integer.parseInt( subTok2, 16 );
        }
        catch( NumberFormatException nfe )
        {
          outputArea.append( "Address and Value must be numeric.\n\n" );
          return;
        }

        nes.getMainMemory().writeNoMessage( param1, param2 );

        String hexVal = Integer.toString( nes.getMainMemory().readNoMessage( param1 ), 16 );
        if( hexVal.length() == 1 )
          hexVal = "0" + hexVal;
        outputArea.append( "Main Memory[$" + subTok1.toUpperCase() + "] set to $" + hexVal.toUpperCase() + "\n" );
      }
      // ------------------------------------------------------------------------------------------
      else if( keyTok.equalsIgnoreCase( "power" ) )
      {
        if( cmdTok.hasMoreTokens() )
        {
          subTok1 = cmdTok.nextToken();

          if( subTok1.equalsIgnoreCase( "on" ) )
          {
            nes.getCpu().powerOn();
            outputArea.append( "CPU Power-On settings loaded.\n" );
          }
          else if( subTok1.equalsIgnoreCase( "off" ) )
          {
            nes.getCpu().powerOff();
            outputArea.append( "CPU Power-Off settings loaded.\n" );
          }
          else
          {
            outputArea.append( "Unknown State: '" + subTok1 + "'\n" );
          }
        }
        else
        {
          outputArea.append( "Unknown State: ''\n" );
        }
      }
      // ------------------------------------------------------------------------------------------
      else if( keyTok.equalsIgnoreCase( "regs" ) )
      {
        outputArea.append( "CPU Registers\n" );
        outputArea.append( "  PC:    $" + Integer.toString( nes.getCpu().getProgramCounterValue(), 16 ).toUpperCase() + "\n" );
        outputArea.append( "  SP:    $" + Integer.toString( nes.getCpu().getStackPointerValue(), 16 ).toUpperCase() + "\n" );
        outputArea.append( "  ACC:   $" + Integer.toString( nes.getCpu().getAccumulatorValue(), 16 ).toUpperCase() + "\n" );
        outputArea.append( "  IX:    $" + Integer.toString( nes.getCpu().getXIndexValue(), 16 ).toUpperCase() + "\n" );
        outputArea.append( "  IY:    $" + Integer.toString( nes.getCpu().getYIndexValue(), 16 ).toUpperCase() + "\n" );

        CpuRegister ps = new CpuRegister();
        ps.setValue( nes.getCpu().getProcessorStatusValue() );
        outputArea.append( "  Flags  S V - B D I Z C\n" );
        outputArea.append( "  PS:    " );
        outputArea.append( String.valueOf( ps.getBitValue( CpuRegister.PS_NEGATIVE_FLAG     ) ) + " " );
        outputArea.append( String.valueOf( ps.getBitValue( CpuRegister.PS_OVERFLOW_FLAG     ) ) + " " );
        outputArea.append( String.valueOf( ps.getBitValue( CpuRegister.PS_UNUSED            ) ) + " " );
        outputArea.append( String.valueOf( ps.getBitValue( CpuRegister.PS_BREAK_COMMAND     ) ) + " " );
        outputArea.append( String.valueOf( ps.getBitValue( CpuRegister.PS_DECIMAL_MODE      ) ) + " " );
        outputArea.append( String.valueOf( ps.getBitValue( CpuRegister.PS_INTERRUPT_DISABLE ) ) + " " );
        outputArea.append( String.valueOf( ps.getBitValue( CpuRegister.PS_ZERO_FLAG         ) ) + " " );
        outputArea.append( String.valueOf( ps.getBitValue( CpuRegister.PS_CARRY_FLAG        ) ) + "\n" );
      }
      // ------------------------------------------------------------------------------------------
      else if( keyTok.equalsIgnoreCase( "op" ) )
      {
        int opCode = -1;
        int param1 = 0;
        int param2 = 0;

        // Load the params
        subTok1 = null;
        subTok2 = null;
        if( cmdTok.hasMoreTokens() )
          subTok1 = cmdTok.nextToken();
        if( cmdTok.hasMoreTokens() )
          subTok2 = cmdTok.nextToken();

        if( subTok1 != null )
        {
          try
          {
            opCode = Integer.parseInt( subTok1, 16 );
          }
          catch( NumberFormatException nfe )
          {
            outputArea.append( "OpCode was not numeric.\n\n" );
            return;
          }

          if( NesOpCodeTable.length[opCode] == 1 )
          {
            // opcodes with no parameters
            outputArea.append( "Syntax Ok: " + NesOpCodeTable.getSyntaxFormatted( opCode, 0, 0 ) + "\n" );
            cycles = nes.getCpu().testExecuteInstruction( opCode, 0, 0 );
            outputArea.append( cycles + " cycles.\n" );
          }
          else if( NesOpCodeTable.length[opCode] > 1 )
          {
            if( subTok2 != null )
            {
              if( NesOpCodeTable.length[opCode] == 2 && subTok2.length() == 2 )
              {
                // opcodes with 1 byte parameters
                try
                {
                  param1 = Integer.parseInt( subTok2, 16 );
                }
                catch( NumberFormatException nfe )
                {
                  outputArea.append( "Params were not numeric.\n\n" );
                  return;
                }

                outputArea.append( "Syntax Ok: " + NesOpCodeTable.getSyntaxFormatted( opCode, param1, 0 ) + "\n" );
                cycles = nes.getCpu().testExecuteInstruction( opCode, param1, 0 );
                outputArea.append( cycles + " cycles.\n" );
              }
              else if( NesOpCodeTable.length[opCode] == 3 && subTok2.length() == 4 )
              {
                // opcodes with 2 byte parameters
                try
                {
                  param1 = Integer.parseInt( subTok2.substring( 0, 2 ), 16 );
                  param2 = Integer.parseInt( subTok2.substring( 2, 4 ), 16 );
                }
                catch( NumberFormatException nfe )
                {
                  outputArea.append( "Params were not numeric.\n\n" );
                  return;
                }

                outputArea.append( "Syntax Ok: " + NesOpCodeTable.getSyntaxFormatted( opCode, param1, param2 ) + "\n" );
                cycles = nes.getCpu().testExecuteInstruction( opCode, param1, param2 );
                outputArea.append( cycles + " cycles.\n" );
              }
              else
              {
                outputArea.append( NesOpCodeTable.name[opCode] + " requires " + (NesOpCodeTable.length[opCode] - 1) + " byte param.\n" );
              }
            }
            else
            {
              outputArea.append( NesOpCodeTable.name[opCode] + " requires " + (NesOpCodeTable.length[opCode] - 1) + " byte param.\n" );
            }
          }
        }
        else
        {
          outputArea.append( "Expected opcode." );
        }
      }
      // ------------------------------------------------------------------------------------------
      else if( keyTok.equalsIgnoreCase( "opx" ) )
      {
        outputArea.append( "opx parser not yet implemented.\n" );
      }
      // ------------------------------------------------------------------------------------------
      else if( keyTok.equalsIgnoreCase( "save" ) )
      {
        if( cmdTok.hasMoreTokens() )
        {
          subTok1 = inCommand.substring( 5 );

          try
          {
            FileOutputStream fos = new FileOutputStream( subTok1 );
            fos.write( outputArea.getText().getBytes() );
            fos.flush();
            fos.close();
            outputArea.append( "Shell Output Saved to: '" + subTok1 + "'.\n" );
          }
          catch( IOException ioe )
          {
            outputArea.append( "Failed to Save Shell Output.\n\n" );
            return;
          }
        }
        else
        {
          outputArea.append( "Expected Script File Name.\n" );
        }
      }
      // ------------------------------------------------------------------------------------------
      else if( keyTok.equalsIgnoreCase( "vidget" ) )
      {
        int param1 = 0;

        // Load the params
        subTok1 = null;
        if( cmdTok.hasMoreTokens() )
          subTok1 = cmdTok.nextToken();
        try
        {
          param1 = Integer.parseInt( subTok1, 16 );
        }
        catch( NumberFormatException nfe )
        {
          outputArea.append( "Address must be numeric.\n\n" );
          return;
        }

        String hexVal = Integer.toString( nes.getVideoMemory().readNoMessage( param1 ), 16 );
        if( hexVal.length() == 1 )
          hexVal = "0" + hexVal;
        outputArea.append( "Video Memory[$" + subTok1.toUpperCase() + "]:  $" + hexVal.toUpperCase() + "\n" );
      }
      // ------------------------------------------------------------------------------------------
      else if( keyTok.equalsIgnoreCase( "vidset" ) )
      {
        int param1 = 0;
        int param2 = 0;

        // Load the params
        subTok1 = null;
        subTok2 = null;
        if( cmdTok.hasMoreTokens() )
          subTok1 = cmdTok.nextToken();
        if( cmdTok.hasMoreTokens() )
          subTok2 = cmdTok.nextToken();
        try
        {
          param1 = Integer.parseInt( subTok1, 16 );
          param2 = Integer.parseInt( subTok2, 16 );
        }
        catch( NumberFormatException nfe )
        {
          outputArea.append( "Address and Value must be numeric.\n\n" );
          return;
        }

        nes.getVideoMemory().writeNoMessage( param1, param2 );

        String hexVal = Integer.toString( nes.getVideoMemory().readNoMessage( param1 ), 16 );
        if( hexVal.length() == 1 )
          hexVal = "0" + hexVal;
        outputArea.append( "Main Memory[$" + subTok1.toUpperCase() + "] set to $" + hexVal.toUpperCase() + "\n" );
      }
      // ------------------------------------------------------------------------------------------
      else
      {
        outputArea.append( "Unknown command: '" + keyTok + "'\n" );
      }

      outputArea.append( "\n" );
    }
  }

  private void initialize()
  {
    outputArea.setText( "MonkeyNES OpCode Test Shell\n" );
    outputArea.append(  "Type 'help' for available commands\n\n" );

    outputArea.append( "Warning:  This test shell replaces $8000-$8002 with the requested\n" );
    outputArea.append( "          OpCode.  No effect of executing the OpCode is restored.\n\n" );
  }

  public void show()
  {
    // Position the window
    Dimension parentSize = parentFrame.getSize();
    Point     parentLoc  = parentFrame.getLocation();
    Dimension mySize     = new Dimension( 500, 400 );
    Point     myLoc      = new Point( parentLoc.x, parentLoc.y );

    // Size and position the window
    mainDialog.setLocation( myLoc );
    mainDialog.setSize( mySize );

    initialize();

    // Show the whole mess...
    mainDialog.show();

    inputText.requestFocus();

    visible = true;
  }

}