package com.monkeycoder.monkeynes.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class OptionsCpuGui
{
  private Frame              parentFrame                      = null;
  private JDialog            mainDialog                       = null;
  
  // Clock speed
  private JPanel             clockSpeedPanel                  = new JPanel( new BorderLayout() );
  private JSlider            clockSpeedSlider                 = new JSlider( JSlider.HORIZONTAL, 250000, 4000000, 1789773 );  // Default to NTSC
  private JLabel             clockSpeedLabel                  = new JLabel( clockSpeedSlider.getValue() + "hz" );
  private JPanel             clockSpeedButtonPanel            = new JPanel( new FlowLayout() );
  private JButton            clockSpeedNtscButton             = new JButton( "NTSC" );
  private JButton            clockSpeedPalButton              = new JButton( "PAL" );
  
  // Buttons
  private JButton            doneButton                       = new JButton( "Done" );
  
  
    
  public OptionsCpuGui( Frame inParentFrame )
  {
    parentFrame = inParentFrame;
    
    // Dialog
    mainDialog = new JDialog( parentFrame, "Options: CPU", false );
    
    // Setup the slider
    clockSpeedSlider.setPaintTicks(true);
    clockSpeedSlider.setSnapToTicks(false);
    clockSpeedSlider.setMajorTickSpacing(250000);
    
    // Build the clock speed panels
    clockSpeedButtonPanel.add( clockSpeedPalButton );
    clockSpeedButtonPanel.add( clockSpeedNtscButton );
    
    clockSpeedPanel.add( GuiHelper.flowWrapper( FlowLayout.RIGHT, clockSpeedLabel ),  BorderLayout.NORTH   );
    clockSpeedPanel.add( new JLabel( "Clock Speed" ),                                 BorderLayout.WEST    );
    clockSpeedPanel.add( clockSpeedSlider,                                            BorderLayout.CENTER  );
    clockSpeedPanel.add( clockSpeedButtonPanel,                                       BorderLayout.EAST    );
    
    // Build the main view
    mainDialog.getContentPane().add( clockSpeedPanel,                                 BorderLayout.NORTH  );
    mainDialog.getContentPane().add( GuiHelper.flowWrapper(doneButton),               BorderLayout.SOUTH  );
    

    // Main window closing listener
    mainDialog.addWindowListener( new WindowAdapter()
    {
      public void windowClosing( WindowEvent we )
      {
        mainDialog.hide();
      }
    } );
    
    // Clock Speed Slider listener
    clockSpeedSlider.addChangeListener( new ChangeListener()
    {
      public void stateChanged( ChangeEvent ce )
      {
        clockSpeedLabel.setText( clockSpeedSlider.getValue() + "hz" );
      }
    } );
    
    // Clock Speed Set NTSC
    clockSpeedNtscButton.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae ) 
      {
        clockSpeedSlider.setValue( 1789773 );
      }
    } );
    
    // Clock Speed Set PAL
    clockSpeedPalButton.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae ) 
      {
        clockSpeedSlider.setValue( 1773447 );
      }
    } );
    
    doneButton.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae ) 
      {
        mainDialog.hide();
      }
    } );
    
  } // End of constructor ----------------------------------------------------------
  
  public long getClockSpeed()
  {
    return (long)clockSpeedSlider.getValue();
  }
  
  
  public void show()
  {
    // Position the window
    Dimension parentSize = parentFrame.getSize();
    Point     parentLoc  = parentFrame.getLocation();
    Dimension mySize     = new Dimension( 400, 150 );
    Point     myLoc      = new Point( (parentLoc.x + (parentSize.width / 2) - (mySize.width / 2 )), 
                                      (parentLoc.y + (parentSize.height / 2) - (mySize.height / 2 )) );
    
    // Size and position the window
    mainDialog.setLocation( myLoc );
    mainDialog.setSize( mySize );
    
    // Show the whole mess...
    mainDialog.show();
  }

}