package com.monkeycoder.monkeynes.gui;

import com.monkeycoder.monkeynes.base.*;
import com.monkeycoder.monkeynes.hardware.*;

import java.awt.*;
import java.awt.image.*;

public class PatternTableView extends Component
{
  private NesVideoMemory     vidMem       = null;
  private int[]              patternTable = null;
  private PatternTableTile[] tiles        = new PatternTableTile[512];

  public PatternTableView( NesVideoMemory inVidMem )
  {
    vidMem = inVidMem;
    buildTiles();
  }
  
  public void buildTiles()
  {
    patternTable = vidMem.readNoMessage( 0x00000000, 0x00001FFF );
    
    int[] top = null;
    int[] bot = null;
    
    int x = 0;
    for( int i = 0; i < patternTable.length; i+=16 )
    {
      top = new int[8];
      bot = new int[8];
      
      for( int j = 0; j < 8; j++ )
      {
        top[j] = patternTable[i + j];
        bot[j] = patternTable[i + j + 8];
      }  
      
      tiles[x] = new PatternTableTile( top, bot );
      x++;
    }
    
    invalidate();
  }
  
  public void paint( Graphics g )
  {
    long startTime = System.currentTimeMillis();
    long stopTime  = 0L;
    
    int[] pix = new int[32768];
    int[] col = new int[4];
    
    // Init my fake color
    col[0] = Color.CYAN.getRGB();
    col[1] = Color.DARK_GRAY.getRGB();
    col[2] = Color.LIGHT_GRAY.getRGB();
    col[3] = Color.WHITE.getRGB();

    // loop across the tiles
    for( int i = 0; i < 16; i++ )
    {
      for( int j = 0; j < 32; j++ )
      { 
        // for each tile, loop across the pixels
        for( int x = 0; x < 8; x++ )
        {
          for( int y = 0; y < 8; y++ )
          { 
            pix[(((j * 8) + y) * 128) + ((i * 8) + x)] = col[tiles[(j * 16) + i].getPixelValue( x, y )];
          }
        }
      }
    }
    g.drawImage( createImage( new MemoryImageSource( 128, 256, pix, 0, 128 ) ), 0, 0, null );      
    
    stopTime = System.currentTimeMillis();
    DebugLog.log( 5, "Pattern Tables Painted: " + (stopTime - startTime) + "ms" );
  }
  
}