package com.monkeycoder.monkeynes.gui;

import com.monkeycoder.monkeynes.base.*;
import com.monkeycoder.monkeynes.hardware.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;


public class PatternTableViewerGui implements Observer
{
  private Frame              parentFrame                      = null;
  private JDialog            mainDialog                       = null;
    
  private NesVideoMemory     vidMem                           = null;    
  private PatternTableView   patternView                      = null;
  
  public PatternTableViewerGui( Frame inParentFrame, NesVideoMemory inVidMemory )
  {
    parentFrame = inParentFrame;
    vidMem      = inVidMemory;
    
    vidMem.addObserver( this );
    
    // Dialog
    mainDialog = new JDialog( parentFrame, "Pattern Tables", false );

    patternView = new PatternTableView( vidMem );
    
    // Build the main view
    mainDialog.getContentPane().add( patternView,     BorderLayout.CENTER  );

    // Main window closing listener
    mainDialog.addWindowListener( new WindowAdapter()
    {
      public void windowClosing( WindowEvent we )
      {
        mainDialog.hide();
      }
    } );
  } // End of constructor ----------------------------------------------------------
  
  
  public void update( Observable o, Object arg )
  {
    MemoryAccessMessage mess = (MemoryAccessMessage)arg;
    
    DebugLog.log( 1, mess.toString() );
    
    if( mess.getMemoryBank() == MemoryAccessMessage.MEM_VIDEO && 
        ( mess.getType() == MemoryAccessMessage.TYPE_WRITE  || 
          mess.getType() == MemoryAccessMessage.TYPE_WRITE_RANGE ) &&
        ( mess.getAddrStart() >= 0x00000000 || 
          mess.getAddrStart() <= 0x00001FFF ) )
    {
      patternView.buildTiles();
      patternView.repaint();
    }
  }
  
  public void show()
  {
    // Position the window
    Dimension parentSize = parentFrame.getSize();
    Point     parentLoc  = parentFrame.getLocation();
    Dimension mySize     = new Dimension( 137, 284 );
    Point     myLoc      = new Point( (parentLoc.x + (parentSize.width / 2) - (mySize.width / 2 )), 
                                      (parentLoc.y + (parentSize.height / 2) - (mySize.height / 2 )) );
    
    // Size and position the window
    mainDialog.setLocation( myLoc );
    mainDialog.setSize( mySize );
    mainDialog.setResizable( false );
    
    // Show the whole mess...
    mainDialog.show();
  }

}