package com.monkeycoder.monkeynes.gui;

import com.monkeycoder.monkeynes.base.*;
import com.monkeycoder.monkeynes.hardware.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;


public class PrgRomViewerGui implements Observer
{
  private Frame              parentFrame                      = null;
  private JDialog            mainDialog                       = null;
    
  private NesMainMemory      mainMem                          = null;    
  private boolean            visible                          = false;
  
  private Document    codeDoc       = new DefaultStyledDocument();
  private JTextArea   codeArea      = new JTextArea( codeDoc, "", 15, 30 );
  private JScrollPane codeScroll    = new JScrollPane( codeArea );
  
  public PrgRomViewerGui( Frame inParentFrame, NesMainMemory inMainMemory )
  {
    parentFrame = inParentFrame;
    mainMem     = inMainMemory;
    
    mainMem.addObserver( this );
    
    codeArea.setFont( new Font( "Courier", Font.PLAIN, 12 ) );
    codeArea.setEditable( false );
    
    // Dialog
    mainDialog = new JDialog( parentFrame, "PRG-ROM Disassembly", false );
    
    // Build the main view
    mainDialog.getContentPane().add( codeScroll,     BorderLayout.CENTER  );

    // Main window closing listener
    mainDialog.addWindowListener( new WindowAdapter()
    {
      public void windowClosing( WindowEvent we )
      {
        mainDialog.hide();
        visible = false;
      }
    } );
  } // End of constructor ----------------------------------------------------------
  
  
  public void update( Observable o, Object arg )
  {
    MemoryAccessMessage mess = (MemoryAccessMessage)arg;
    
    DebugLog.log( 1, mess.toString() );
    
    if( visible && 
        mess.getMemoryBank() == MemoryAccessMessage.MEM_MAIN && 
        ( mess.getType() == MemoryAccessMessage.TYPE_WRITE  || 
          mess.getType() == MemoryAccessMessage.TYPE_WRITE_RANGE ) &&
        ( mess.getAddrStart() >= 0x00008000 || 
          mess.getAddrStart() <= 0x0000FFFF ) )
    {
      disassembleCode();
    }
  }
  
  private void disassembleCode()
  {
    StringBuffer sb = new StringBuffer();
    int opCodeNum = 0;
    
    sb.append( "NOTE: This is a simple top-down\n" );
    sb.append( "      disassembly of the PRG-ROM.\n" );
    sb.append( "      For that reason parts of it\n" );
    sb.append( "      are likely inaccurate.\n\n" );
    
    for( int i = 0x00008000; i <= 0x0000FFFF; )
    {
      opCodeNum = mainMem.readNoMessage( i );
      
      // Output the address
      sb.append( "$" );
      sb.append( MathHelper.getHexString16( i ) );
      sb.append( ": " );
 
      // Output the opcode name
      if( NesOpCodeTable.name[opCodeNum].equals( "---" ) )
      {
        sb.append( "---  Op: $" );
        sb.append( MathHelper.getHexString8(opCodeNum) );
      }
      else
      {
        switch( NesOpCodeTable.length[opCodeNum] )
        {
          case 1:
            sb.append( NesOpCodeTable.getSyntaxFormatted( opCodeNum, 0, 0 ) );
            break;
          case 2:
            sb.append( NesOpCodeTable.getSyntaxFormatted( opCodeNum, mainMem.readNoMessage( i + 1 ), 0 ) );
            break;
          case 3:
            sb.append( NesOpCodeTable.getSyntaxFormatted( opCodeNum, mainMem.readNoMessage( i + 1 ), mainMem.readNoMessage( i + 2 ) ) );
            break;
        }    
      }  
        
      sb.append( "\n" );
        
      i += NesOpCodeTable.length[opCodeNum];
    }
    
    codeArea.setText( "" );
    codeArea.append( sb.toString() );
  }
  
  public void show()
  {
    // Position the window
    Dimension parentSize = parentFrame.getSize();
    Point     parentLoc  = parentFrame.getLocation();
    Dimension mySize     = new Dimension( 300, 400 );
    Point     myLoc      = new Point( (parentLoc.x + (parentSize.width / 2) - (mySize.width / 2 )), 
                                      (parentLoc.y + (parentSize.height / 2) - (mySize.height / 2 )) );
    
    // Size and position the window
    mainDialog.setLocation( myLoc );
    mainDialog.setSize( mySize );
 
    disassembleCode();
    
    // Show the whole mess...
    mainDialog.show();
    
    visible = true;
  }

}