package com.monkeycoder.monkeynes.hardware;

public class CpuRegister
{
  // Known status register bitmasks
  public static final int PS_CARRY_FLAG           = 0x00000001;    // C
  public static final int PS_ZERO_FLAG            = 0x00000002;    // Z
  public static final int PS_INTERRUPT_DISABLE    = 0x00000004;    // I
  public static final int PS_DECIMAL_MODE         = 0x00000008;    // D - Not supported by NES
  public static final int PS_BREAK_COMMAND        = 0x00000010;    // B
  public static final int PS_UNUSED               = 0x00000020;    // not used - supposed to be 1
  public static final int PS_OVERFLOW_FLAG        = 0x00000040;    // V
  public static final int PS_NEGATIVE_FLAG        = 0x00000080;    // S
  
  private int data = 0;

  public CpuRegister()
  {
    data = 0;
  }

  public int getValue()
  {
    return data;
  }

  public boolean getBooleanBitValue( int inMask )
  {
    return ((data & inMask) == inMask);
  }
  
  public int getBitValue( int inMask )
  {
    return ( (getBooleanBitValue( inMask )) ? (1) : (0) );
  }  
  
  public void setValue( int inVal )
  {
    data = inVal;
  }
  
  // 0 or 1+ version
  public void setBitValue( int inMask, int inVal )
  {
    if( inVal != 0 )
      data |= inMask;
    else
      data &= ~inMask;
  }
  
  // true or false version
  public void setBitValue( int inMask, boolean inVal )
  {
    if( inVal )
      data |= inMask;
    else
      data &= ~inMask;
  }
  
  public void setBitOn( int inMask )
  {
    data |= inMask;
  }  
  
  public void setBitOff( int inMask )
  {
    data &= ~inMask;
  }
  
  public void increment()
  {
    data++;
  }
  
  public void plusEquals( int inVal )
  {
    data += inVal;
  }
  
  public void decrement()
  {
    data--;
  }
  
  public void minusEquals( int inVal )
  {
    data -= inVal;
  }
  
  public void andEquals( int inVal )
  {
    data &= inVal;
  }
  
  public void orEquals( int inVal )
  {
    data |= inVal;
  }
  
  public void xorEquals( int inVal )
  {
    data ^= inVal;
  }
  
  public void wrapValue( int numBits )
  {
    data %= (0x00000001 << numBits);
  }
  
}