package com.monkeycoder.monkeynes.hardware;

import com.monkeycoder.monkeynes.base.*;

public class NesOpCodeTable
{
  public static final int MODE_UNKNOWN             = 0;   // For unknown opcodes
  public static final int MODE_IMPLIED             = 1;
  public static final int MODE_ACCUMULATOR         = 2;
  public static final int MODE_IMMEDIATE           = 3;
  public static final int MODE_ZERO_PAGE           = 4;
  public static final int MODE_ZERO_PAGE_X         = 5;
  public static final int MODE_ZERO_PAGE_Y         = 6;
  public static final int MODE_ABSOLUTE            = 7;
  public static final int MODE_ABSOLUTE_X          = 8;
  public static final int MODE_ABSOLUTE_Y          = 9;
  public static final int MODE_INDIRECT            = 10;
  public static final int MODE_INDIRECT_X          = 11;
  public static final int MODE_INDIRECT_Y          = 12;
  public static final int MODE_RELATIVE            = 13;
  
  // ---------------------------------------------------------------------------
  
  // For tokenizing the mode syntax table entries
  private static final String SYNTAX_OPCODE       = "[op]";
  private static final String SYNTAX_PAGEADDRESS  = "[pa]";
  private static final String SYNTAX_ADDRESS      = "[ad]";
    
  private static String[] modeSyntax = { "",                    // MODE_UNKNOWN       
                                         "[op]",                // MODE_IMPLIED    
                                         "[op] A",              // MODE_ACCUMULATOR
                                         "[op] #$[pa]",         // MODE_IMMEDIATE  
                                         "[op] $[pa]",          // MODE_ZERO_PAGE   (Zero Page Absolute)
                                         "[op] $[pa],X",        // MODE_ZERO_PAGE_X (Zero Page Indexed)
                                         "[op] $[pa],Y",        // MODE_ZERO_PAGE_Y (Zero Page Indexed)
                                         "[op] $[ad]",          // MODE_ABSOLUTE   
                                         "[op] $[ad],X",        // MODE_ABSOLUTE_X  (Indexed)
                                         "[op] $[ad],Y",        // MODE_ABSOLUTE_Y  (Indexed)
                                         "[op] ($[ad])",        // MODE_INDIRECT   
                                         "[op] ($[pa],X)",      // MODE_INDIRECT_X  (Pre-Indexed Indirect)
                                         "[op] ($[pa]),Y",      // MODE_INDIRECT_Y  (Post-Indexed Indirect)
                                         "[op] $[pa]"           // MODE_RELATIVE 
                                       };
                                
  public static String getSyntaxFormatted( int opCodeNum, int inByte1, int inByte2 )
  {
    String syntax = modeSyntax[mode[opCodeNum]];
    String out    = modeSyntax[mode[opCodeNum]];
    
    // OpCode
    if( out.indexOf( SYNTAX_OPCODE ) != -1 )
      out = out.substring( 0, out.indexOf( SYNTAX_OPCODE ) ) + name[opCodeNum] + out.substring( out.indexOf( SYNTAX_OPCODE ) + SYNTAX_OPCODE.length() );

    // PageAddress 
    if( out.indexOf( SYNTAX_PAGEADDRESS ) != -1 )
    {
      String pa = MathHelper.getHexString8( inByte1 );
      out = out.substring( 0, out.indexOf( SYNTAX_PAGEADDRESS ) ) + pa + out.substring( out.indexOf( SYNTAX_PAGEADDRESS ) + SYNTAX_PAGEADDRESS.length() );
    } 
    
    // Address
    if( out.indexOf( SYNTAX_ADDRESS ) != -1 )
    {
      String ad = MathHelper.getHexString8( inByte2 ) + MathHelper.getHexString8( inByte1 );
      out = out.substring( 0, out.indexOf( SYNTAX_ADDRESS ) ) + ad + out.substring( out.indexOf( SYNTAX_ADDRESS ) + SYNTAX_ADDRESS.length() );
    }
    
    return out; 
  }                                
                                
  // ---------------------------------------------------------------------------                                
  
  public static final String[] name =
  {
    "BRK",                     // $00 - BRK (Implied)
    "ORA",                     // $01 - ORA (Indirect,X)
    "---",                     // $02 -
    "---",                     // $03 -
    "---",                     // $04 -
    "ORA",                     // $05 - ORA (Zero Page)
    "ASL",                     // $06 - ASL (Zero Page)
    "---",                     // $07 -
    "PHP",                     // $08 - PHP (Implied)
    "ORA",                     // $09 - ORA (Immediate)
    "ASL",                     // $0A - ASL (Accumulator)
    "---",                     // $0B -
    "---",                     // $0C -
    "ORA",                     // $0D - ORA (Absolute)
    "ASL",                     // $0E - ASL (Absolute)
    "---",                     // $0F -
    "BPL",                     // $10 - BPL (Relative)
    "ORA",                     // $11 - ORA (Indirect,Y)
    "---",                     // $12 -
    "---",                     // $13 -
    "---",                     // $14 -
    "ORA",                     // $15 - ORA (Zero Page,X)
    "ASL",                     // $16 - ASL (Zero Page,X)
    "---",                     // $17 -
    "CLC",                     // $18 - CLC (Implied)
    "ORA",                     // $19 - ORA (Absolute,Y)
    "---",                     // $1A -
    "---",                     // $1B -
    "---",                     // $1C -
    "ORA",                     // $1D - ORA (Absolute,X)
    "ASL",                     // $1E - ASL (Absolute,X)
    "---",                     // $1F -
    "JSR",                     // $20 - JSR (Absolute)
    "AND",                     // $21 - AND (Indirect,X)
    "---",                     // $22 -
    "---",                     // $23 -
    "BIT",                     // $24 - BIT (Zero Page)
    "AND",                     // $25 - AND (Zero Page)
    "ROL",                     // $26 - ROL (Zero Page)
    "---",                     // $27 -
    "PLP",                     // $28 - PLP (Implied)
    "AND",                     // $29 - AND (Immediate)
    "ROL",                     // $2A - ROL (Accumulator)
    "---",                     // $2B -
    "BIT",                     // $2C - BIT (Absolute)
    "AND",                     // $2D - AND (Absolute)
    "ROL",                     // $2E - ROL (Absolute)
    "---",                     // $2F -
    "BMI",                     // $30 - BMI (Relative)
    "AND",                     // $31 - AND (Indirect,Y)
    "---",                     // $32 -
    "---",                     // $33 -
    "---",                     // $34 -
    "AND",                     // $35 - AND (Zero Page,X)
    "ROL",                     // $36 - ROL (Zero Page,X)
    "---",                     // $37 -
    "SEC",                     // $38 - SEC (Implied)
    "AND",                     // $39 - AND (Absolute,Y)
    "---",                     // $3A -
    "---",                     // $3B -
    "---",                     // $3C -
    "AND",                     // $3D - AND (Absolute,X)
    "ROL",                     // $3E - ROL (Absolute,X)
    "---",                     // $3F -
    "RTI",                     // $40 - RTI (Implied)
    "EOR",                     // $41 - EOR (Indirect,X)
    "---",                     // $42 -
    "---",                     // $43 -
    "---",                     // $44 -
    "EOR",                     // $45 - EOR (Zero Page)
    "LSR",                     // $46 - LSR (Zero Page)
    "---",                     // $47 -
    "PHA",                     // $48 - PHA (Implied)
    "EOR",                     // $49 - EOR (Immediate)
    "LSR",                     // $4A - LSR (Accumulator)
    "---",                     // $4B -
    "JMP",                     // $4C - JMP (Absolute)
    "EOR",                     // $4D - EOR (Absolute)
    "LSR",                     // $4E - LSR (Absolute)
    "---",                     // $4F -
    "BVC",                     // $50 - BVC (Relative)
    "EOR",                     // $51 - EOR (Indirect,Y)
    "---",                     // $52 -
    "---",                     // $53 -
    "---",                     // $54 -
    "EOR",                     // $55 - EOR (Zero Page,X)
    "LSR",                     // $56 - LSR (Zero Page,X)
    "---",                     // $57 -
    "CLI",                     // $58 - CLI (Implied)
    "EOR",                     // $59 - EOR (Absolute,Y)
    "---",                     // $5A -
    "---",                     // $5B -
    "---",                     // $5C -
    "EOR",                     // $5D - EOR (Absolute,X)
    "LSR",                     // $5E - LSR (Absolute,X)
    "---",                     // $5F -
    "RTS",                     // $60 - RTS (Implied)
    "ADC",                     // $61 - ADC (Indirect,X)
    "---",                     // $62 -
    "---",                     // $63 -
    "---",                     // $64 -
    "ADC",                     // $65 - ADC (Zero Page)
    "ROR",                     // $66 - ROR (Zero Page)
    "---",                     // $67 -
    "PLA",                     // $68 - PLA (Implied)
    "ADC",                     // $69 - ADC (Immediate)
    "ROR",                     // $6A - ROR (Accumulator)
    "---",                     // $6B -
    "JMP",                     // $6C - JMP (Indirect)
    "ADC",                     // $6D - ADC (Absolute)
    "ROR",                     // $6E - ROR (Absolute)
    "---",                     // $6F -
    "BVS",                     // $70 - BVS (Relative)
    "ADC",                     // $71 - ADC (Indirect,Y)
    "---",                     // $72 - 
    "---",                     // $73 -
    "---",                     // $74 -
    "ADC",                     // $75 - ADC (Zero Page,X)
    "ROR",                     // $76 - ROR (Zero Page,X)
    "---",                     // $77 -
    "SEI",                     // $78 - SEI (Implied)
    "ADC",                     // $79 - ADC (Absolute,Y)
    "---",                     // $7A -
    "---",                     // $7B -
    "JMP",                     // $7C - JMP (Absolute,X)
    "ADC",                     // $7D - ADC (Absolute,X)
    "ROR",                     // $7E - ROR (Absolute,X)
    "---",                     // $7F -
    "---",                     // $80 -
    "STA",                     // $81 - STA (Indirect,X)
    "---",                     // $82 -
    "---",                     // $83 -
    "STY",                     // $84 - STY (Zero Page)
    "STA",                     // $85 - STA (Zero Page)
    "STX",                     // $86 - STX (Zero Page)
    "---",                     // $87 -
    "DEY",                     // $88 - DEY (Implied)
    "---",                     // $89 -
    "TXA",                     // $8A - TXA (Implied)
    "---",                     // $8B -
    "STY",                     // $8C - STY (Absolute)
    "STA",                     // $8D - STA (Absolute)
    "STX",                     // $8E - STX (Absolute)
    "---",                     // $8F -
    "BCC",                     // $90 - BCC (Relative)
    "STA",                     // $91 - STA (Indirect,Y)
    "---",                     // $92 -
    "---",                     // $93 -
    "STY",                     // $94 - STY (Zero Page,X)
    "STA",                     // $95 - STA (Zero Page,X)
    "STX",                     // $96 - STX (Zero Page,Y)
    "---",                     // $97 -
    "TYA",                     // $98 - TYA (Implied)
    "STA",                     // $99 - STA (Absolute,Y)
    "TXS",                     // $9A - TXS (Implied)
    "---",                     // $9B -
    "---",                     // $9C -
    "STA",                     // $9D - STA (Absolute,X)
    "---",                     // $9E -
    "---",                     // $9F -
    "LDY",                     // $A0 - LDY (Immediate)
    "LDA",                     // $A1 - LDA (Indirect,X)
    "LDX",                     // $A2 - LDX (Immediate)
    "---",                     // $A3 -
    "LDY",                     // $A4 - LDY (Zero Page)
    "LDA",                     // $A5 - LDA (Zero Page)
    "LDX",                     // $A6 - LDX (Zero Page)
    "---",                     // $A7 -
    "TAY",                     // $A8 - TAY (Implied)
    "LDA",                     // $A9 - LDA (Immediate)
    "TAX",                     // $AA - TAX (Implied)
    "---",                     // $AB -
    "LDY",                     // $AC - LDY (Absolute)
    "LDA",                     // $AD - LDA (Absolute)
    "LDX",                     // $AE - LDX (Absolute)
    "---",                     // $AF -
    "BCS",                     // $B0 - BCS (Relative)
    "LDA",                     // $B1 - LDA (Indirect,Y)
    "---",                     // $B2 -
    "---",                     // $B3 -
    "LDY",                     // $B4 - LDY (Zero Page,X)
    "LDA",                     // $B5 - LDA (Zero Page,X)
    "LDX",                     // $B6 - LDX (Zero Page,Y)
    "---",                     // $B7 -
    "CLV",                     // $B8 - CLV (Implied)
    "LDA",                     // $B9 - LDA (Absolute,Y)
    "TSX",                     // $BA - TSX (Implied)
    "---",                     // $BB -
    "LDY",                     // $BC - LDY (Absolute,X)
    "LDA",                     // $BD - LDA (Absolute,X)
    "LDX",                     // $BE - LDX (Absolute,Y)
    "---",                     // $BF -
    "CPY",                     // $C0 - CPY (Immediate)
    "CMP",                     // $C1 - CMP (Indirect,X)
    "---",                     // $C2 -
    "---",                     // $C3 -
    "CPY",                     // $C4 - CPY (Zero Page)
    "CMP",                     // $C5 - CMP (Zero Page)
    "DEC",                     // $C6 - DEC (Zero Page)
    "---",                     // $C7 -
    "INY",                     // $C8 - INY (Implied)
    "CMP",                     // $C9 - CMP (Immediate)
    "DEX",                     // $CA - DEX (Implied)
    "---",                     // $CB -
    "CPY",                     // $CC - CPY (Absolute)
    "CMP",                     // $CD - CMP (Absolute)
    "DEC",                     // $CE - DEC (Absolute)
    "---",                     // $CF -
    "BNE",                     // $D0 - BNE (Relative)
    "CMP",                     // $D1 - CMP (Indirect,Y)
    "---",                     // $D2 -
    "---",                     // $D3 -
    "---",                     // $D4 -
    "CMP",                     // $D5 - CMP (Zero Page,X)
    "DEC",                     // $D6 - DEC (Zero Page,X)
    "---",                     // $D7 -
    "CLD",                     // $D8 - CLD (Implied)
    "CMP",                     // $D9 - CMP (Absolute,Y)
    "---",                     // $DA -
    "---",                     // $DB -
    "---",                     // $DC -
    "CMP",                     // $DD - CMP (Absolute,X)
    "DEC",                     // $DE - DEC (Absolute,X)
    "---",                     // $DF -
    "CPX",                     // $E0 - CPX (Immediate)
    "SBC",                     // $E1 - SBC (Indirect,X)
    "---",                     // $E2 -
    "---",                     // $E3 -
    "CPX",                     // $E4 - CPX (Zero Page)
    "SBC",                     // $E5 - SBC (Zero Page)
    "INC",                     // $E6 - INC (Zero Page)
    "---",                     // $E7 -
    "INX",                     // $E8 - INX (Implied)
    "SBC",                     // $E9 - SBC (Immediate)
    "NOP",                     // $EA - NOP (Implied)
    "---",                     // $EB -
    "CPX",                     // $EC - CPX (Absolute)
    "SBC",                     // $ED - SBC (Absolute)
    "INC",                     // $EE - INC (Absolute)
    "---",                     // $EF -
    "BEQ",                     // $F0 - BEQ (Relative)
    "SBC",                     // $F1 - SBC (Indirect,Y)
    "---",                     // $F2 -
    "---",                     // $F3 -
    "---",                     // $F4 -
    "SBC",                     // $F5 - SBC (Zero Page,X)
    "INC",                     // $F6 - INC (Zero Page,X)
    "---",                     // $F7 -
    "SED",                     // $F8 - SED (Implied)
    "SBC",                     // $F9 - SBC (Absolute,Y)
    "---",                     // $FA -
    "---",                     // $FB -
    "---",                     // $FC -
    "SBC",                     // $FD - SBC (Absolute,X)
    "INC",                     // $FE - INC (Absolute,X)
    "---"                      // $FF -
  };
   
  // ---------------------------------------------------------------------------
   
  public static final int[] mode =
  {
    MODE_IMPLIED,              // $00 - BRK (Implied)
    MODE_INDIRECT_X,           // $01 - ORA (Indirect,X)
    MODE_UNKNOWN,              // $02 -
    MODE_UNKNOWN,              // $03 -
    MODE_UNKNOWN,              // $04 -
    MODE_ZERO_PAGE,            // $05 - ORA (Zero Page)
    MODE_ZERO_PAGE,            // $06 - ASL (Zero Page)
    MODE_UNKNOWN,              // $07 -
    MODE_IMPLIED,              // $08 - PHP (Implied)
    MODE_IMMEDIATE,            // $09 - ORA (Immediate)
    MODE_ACCUMULATOR,          // $0A - ASL (Accumulator)
    MODE_UNKNOWN,              // $0B -
    MODE_UNKNOWN,              // $0C -
    MODE_ABSOLUTE,             // $0D - ORA (Absolute)
    MODE_ABSOLUTE,             // $0E - ASL (Absolute)
    MODE_UNKNOWN,              // $0F -
    MODE_RELATIVE,             // $10 - BPL (Relative)
    MODE_INDIRECT_Y,           // $11 - ORA (Indirect,Y)
    MODE_UNKNOWN,              // $12 -
    MODE_UNKNOWN,              // $13 -
    MODE_UNKNOWN,              // $14 -
    MODE_ZERO_PAGE_X,          // $15 - ORA (Zero Page,X)
    MODE_ZERO_PAGE_X,          // $16 - ASL (Zero Page,X)
    MODE_UNKNOWN,              // $17 -
    MODE_IMPLIED,              // $18 - CLC (Implied)
    MODE_ABSOLUTE_Y,           // $19 - ORA (Absolute,Y)
    MODE_UNKNOWN,              // $1A -
    MODE_UNKNOWN,              // $1B -
    MODE_UNKNOWN,              // $1C -
    MODE_ABSOLUTE_X,           // $1D - ORA (Absolute,X)
    MODE_ABSOLUTE_X,           // $1E - ASL (Absolute,X)
    MODE_UNKNOWN,              // $1F -
    MODE_ABSOLUTE,             // $20 - JSR (Absolute)
    MODE_INDIRECT_X,           // $21 - AND (Indirect,X)
    MODE_UNKNOWN,              // $22 -
    MODE_UNKNOWN,              // $23 -
    MODE_ZERO_PAGE,            // $24 - BIT (Zero Page)
    MODE_ZERO_PAGE,            // $25 - AND (Zero Page)
    MODE_ZERO_PAGE,            // $26 - ROL (Zero Page)
    MODE_UNKNOWN,              // $27 -
    MODE_IMPLIED,              // $28 - PLP (Implied)
    MODE_IMMEDIATE,            // $29 - AND (Immediate)
    MODE_ACCUMULATOR,          // $2A - ROL (Accumulator)
    MODE_UNKNOWN,              // $2B -
    MODE_ABSOLUTE,             // $2C - BIT (Absolute)
    MODE_ABSOLUTE,             // $2D - AND (Absolute)
    MODE_ABSOLUTE,             // $2E - ROL (Absolute)
    MODE_UNKNOWN,              // $2F -
    MODE_RELATIVE,             // $30 - BMI (Relative)
    MODE_INDIRECT_Y,           // $31 - AND (Indirect,Y)
    MODE_UNKNOWN,              // $32 -
    MODE_UNKNOWN,              // $33 -
    MODE_UNKNOWN,              // $34 -
    MODE_ZERO_PAGE_X,          // $35 - AND (Zero Page,X)
    MODE_ZERO_PAGE_X,          // $36 - ROL (Zero Page,X)
    MODE_UNKNOWN,              // $37 -
    MODE_IMPLIED,              // $38 - SEC (Implied)
    MODE_ABSOLUTE_Y,           // $39 - AND (Absolute,Y)
    MODE_UNKNOWN,              // $3A -
    MODE_UNKNOWN,              // $3B -
    MODE_UNKNOWN,              // $3C -
    MODE_ABSOLUTE_X,           // $3D - AND (Absolute,X)
    MODE_ABSOLUTE_X,           // $3E - ROL (Absolute,X)
    MODE_UNKNOWN,              // $3F -
    MODE_IMPLIED,              // $40 - RTI (Implied)
    MODE_INDIRECT_X,           // $41 - EOR (Indirect,X)
    MODE_UNKNOWN,              // $42 -
    MODE_UNKNOWN,              // $43 -
    MODE_UNKNOWN,              // $44 -
    MODE_ZERO_PAGE,            // $45 - EOR (Zero Page)
    MODE_ZERO_PAGE,            // $46 - LSR (Zero Page)
    MODE_UNKNOWN,              // $47 -
    MODE_IMPLIED,              // $48 - PHA (Implied)
    MODE_IMMEDIATE,            // $49 - EOR (Immediate)
    MODE_ACCUMULATOR,          // $4A - LSR (Accumulator)
    MODE_UNKNOWN,              // $4B -
    MODE_ABSOLUTE,             // $4C - JMP (Absolute)
    MODE_ABSOLUTE,             // $4D - EOR (Absolute)
    MODE_ABSOLUTE,             // $4E - LSR (Absolute)
    MODE_UNKNOWN,              // $4F -
    MODE_RELATIVE,             // $50 - BVC (Relative)
    MODE_INDIRECT_Y,           // $51 - EOR (Indirect,Y)
    MODE_UNKNOWN,              // $52 -
    MODE_UNKNOWN,              // $53 -
    MODE_UNKNOWN,              // $54 -
    MODE_ZERO_PAGE_X,          // $55 - EOR (Zero Page,X)
    MODE_ZERO_PAGE_X,          // $56 - LSR (Zero Page,X)
    MODE_UNKNOWN,              // $57 -
    MODE_IMPLIED,              // $58 - CLI (Implied)
    MODE_ABSOLUTE_Y,           // $59 - EOR (Absolute,Y)
    MODE_UNKNOWN,              // $5A -
    MODE_UNKNOWN,              // $5B -
    MODE_UNKNOWN,              // $5C -
    MODE_ABSOLUTE_X,           // $5D - EOR (Absolute,X)
    MODE_ABSOLUTE_X,           // $5E - LSR (Absolute,X)
    MODE_UNKNOWN,              // $5F -
    MODE_IMPLIED,              // $60 - RTS (Implied)
    MODE_INDIRECT_X,           // $61 - ADC (Indirect,X)
    MODE_UNKNOWN,              // $62 -
    MODE_UNKNOWN,              // $63 -
    MODE_UNKNOWN,              // $64 -
    MODE_ZERO_PAGE,            // $65 - ADC (Zero Page)
    MODE_ZERO_PAGE,            // $66 - ROR (Zero Page)
    MODE_UNKNOWN,              // $67 -
    MODE_IMPLIED,              // $68 - PLA (Implied)
    MODE_IMMEDIATE,            // $69 - ADC (Immediate)
    MODE_ACCUMULATOR,          // $6A - ROR (Accumulator)
    MODE_UNKNOWN,              // $6B -
    MODE_INDIRECT,             // $6C - JMP (Indirect)
    MODE_ABSOLUTE,             // $6D - ADC (Absolute)
    MODE_ABSOLUTE,             // $6E - ROR (Absolute)
    MODE_UNKNOWN,              // $6F -
    MODE_RELATIVE,             // $70 - BVS (Relative)
    MODE_INDIRECT_Y,           // $71 - ADC (Indirect,Y)
    MODE_UNKNOWN,              // $72 - 
    MODE_UNKNOWN,              // $73 -
    MODE_UNKNOWN,              // $74 -
    MODE_ZERO_PAGE_X,          // $75 - ADC (Zero Page,X)
    MODE_ZERO_PAGE_X,          // $76 - ROR (Zero Page,X)
    MODE_UNKNOWN,              // $77 -
    MODE_IMPLIED,              // $78 - SEI (Implied)
    MODE_ABSOLUTE_Y,           // $79 - ADC (Absolute,Y)
    MODE_UNKNOWN,              // $7A -
    MODE_UNKNOWN,              // $7B -
    MODE_ABSOLUTE_X,           // $7C - JMP (Absolute,X)
    MODE_ABSOLUTE_X,           // $7D - ADC (Absolute,X)
    MODE_ABSOLUTE_X,           // $7E - ROR (Absolute,X)
    MODE_UNKNOWN,              // $7F -
    MODE_UNKNOWN,              // $80 -
    MODE_INDIRECT_X,           // $81 - STA (Indirect,X)
    MODE_UNKNOWN,              // $82 -
    MODE_UNKNOWN,              // $83 -
    MODE_ZERO_PAGE,            // $84 - STY (Zero Page)
    MODE_ZERO_PAGE,            // $85 - STA (Zero Page)
    MODE_ZERO_PAGE,            // $86 - STX (Zero Page)
    MODE_UNKNOWN,              // $87 -
    MODE_IMPLIED,              // $88 - DEY (Implied)
    MODE_UNKNOWN,              // $89 -
    MODE_IMPLIED,              // $8A - TXA (Implied)
    MODE_UNKNOWN,              // $8B -
    MODE_ABSOLUTE,             // $8C - STY (Absolute)
    MODE_ABSOLUTE,             // $8D - STA (Absolute)
    MODE_ABSOLUTE,             // $8E - STX (Absolute)
    MODE_UNKNOWN,              // $8F -
    MODE_RELATIVE,             // $90 - BCC (Relative)
    MODE_INDIRECT_Y,           // $91 - STA (Indirect,Y)
    MODE_UNKNOWN,              // $92 -
    MODE_UNKNOWN,              // $93 -
    MODE_ZERO_PAGE_X,          // $94 - STY (Zero Page,X)
    MODE_ZERO_PAGE_X,          // $95 - STA (Zero Page,X)
    MODE_ZERO_PAGE_Y,          // $96 - STX (Zero Page,Y)
    MODE_UNKNOWN,              // $97 -
    MODE_IMPLIED,              // $98 - TYA (Implied)
    MODE_ABSOLUTE_Y,           // $99 - STA (Absolute,Y)
    MODE_IMPLIED,              // $9A - TXS (Implied)
    MODE_UNKNOWN,              // $9B -
    MODE_UNKNOWN,              // $9C -
    MODE_ABSOLUTE_X,           // $9D - STA (Absolute,X)
    MODE_UNKNOWN,              // $9E -
    MODE_UNKNOWN,              // $9F -
    MODE_IMMEDIATE,            // $A0 - LDY (Immediate)
    MODE_INDIRECT_X,           // $A1 - LDA (Indirect,X)
    MODE_IMMEDIATE,            // $A2 - LDX (Immediate)
    MODE_UNKNOWN,              // $A3 -
    MODE_ZERO_PAGE,            // $A4 - LDY (Zero Page)
    MODE_ZERO_PAGE,            // $A5 - LDA (Zero Page)
    MODE_ZERO_PAGE,            // $A6 - LDX (Zero Page)
    MODE_UNKNOWN,              // $A7 -
    MODE_IMPLIED,              // $A8 - TAY (Implied)
    MODE_IMMEDIATE,            // $A9 - LDA (Immediate)
    MODE_IMPLIED,              // $AA - TAX (Implied)
    MODE_UNKNOWN,              // $AB -
    MODE_ABSOLUTE,             // $AC - LDY (Absolute)
    MODE_ABSOLUTE,             // $AD - LDA (Absolute)
    MODE_ABSOLUTE,             // $AE - LDX (Absolute)
    MODE_UNKNOWN,              // $AF -
    MODE_RELATIVE,             // $B0 - BCS (Relative)
    MODE_INDIRECT_Y,           // $B1 - LDA (Indirect,Y)
    MODE_UNKNOWN,              // $B2 -
    MODE_UNKNOWN,              // $B3 -
    MODE_ZERO_PAGE_X,          // $B4 - LDY (Zero Page,X)
    MODE_ZERO_PAGE_X,          // $B5 - LDA (Zero Page,X)
    MODE_ZERO_PAGE_Y,          // $B6 - LDX (Zero Page,Y)
    MODE_UNKNOWN,              // $B7 -
    MODE_IMPLIED,              // $B8 - CLV (Implied)
    MODE_ABSOLUTE_Y,           // $B9 - LDA (Absolute,Y)
    MODE_IMPLIED,              // $BA - TSX (Implied)
    MODE_UNKNOWN,              // $BB -
    MODE_ABSOLUTE_X,           // $BC - LDY (Absolute,X)
    MODE_ABSOLUTE_X,           // $BD - LDA (Absolute,X)
    MODE_ABSOLUTE_Y,           // $BE - LDX (Absolute,Y)
    MODE_UNKNOWN,              // $BF -
    MODE_IMMEDIATE,            // $C0 - CPY (Immediate)
    MODE_INDIRECT_X,           // $C1 - CMP (Indirect,X)
    MODE_UNKNOWN,              // $C2 -
    MODE_UNKNOWN,              // $C3 -
    MODE_ZERO_PAGE,            // $C4 - CPY (Zero Page)
    MODE_ZERO_PAGE,            // $C5 - CMP (Zero Page)
    MODE_ZERO_PAGE,            // $C6 - DEC (Zero Page)
    MODE_UNKNOWN,              // $C7 -
    MODE_IMPLIED,              // $C8 - INY (Implied)
    MODE_IMMEDIATE,            // $C9 - CMP (Immediate)
    MODE_IMPLIED,              // $CA - DEX (Implied)
    MODE_UNKNOWN,              // $CB -
    MODE_ABSOLUTE,             // $CC - CPY (Absolute)
    MODE_ABSOLUTE,             // $CD - CMP (Absolute)
    MODE_ABSOLUTE,             // $CE - DEC (Absolute)
    MODE_UNKNOWN,              // $CF -
    MODE_RELATIVE,             // $D0 - BNE (Relative)
    MODE_INDIRECT_Y,           // $D1 - CMP (Indirect,Y)
    MODE_UNKNOWN,              // $D2 -
    MODE_UNKNOWN,              // $D3 -
    MODE_UNKNOWN,              // $D4 -
    MODE_ZERO_PAGE_X,          // $D5 - CMP (Zero Page,X)
    MODE_ZERO_PAGE_X,          // $D6 - DEC (Zero Page,X)
    MODE_UNKNOWN,              // $D7 -
    MODE_IMPLIED,              // $D8 - CLD (Implied)
    MODE_ABSOLUTE_Y,           // $D9 - CMP (Absolute,Y)
    MODE_UNKNOWN,              // $DA -
    MODE_UNKNOWN,              // $DB -
    MODE_UNKNOWN,              // $DC -
    MODE_ABSOLUTE_X,           // $DD - CMP (Absolute,X)
    MODE_ABSOLUTE_X,           // $DE - DEC (Absolute,X)
    MODE_UNKNOWN,              // $DF -
    MODE_IMMEDIATE,            // $E0 - CPX (Immediate)
    MODE_INDIRECT_X,           // $E1 - SBC (Indirect,X)
    MODE_UNKNOWN,              // $E2 -
    MODE_UNKNOWN,              // $E3 -
    MODE_ZERO_PAGE,            // $E4 - CPX (Zero Page)
    MODE_ZERO_PAGE,            // $E5 - SBC (Zero Page)
    MODE_ZERO_PAGE,            // $E6 - INC (Zero Page)
    MODE_UNKNOWN,              // $E7 -
    MODE_IMPLIED,              // $E8 - INX (Implied)
    MODE_IMMEDIATE,            // $E9 - SBC (Immediate)
    MODE_IMPLIED,              // $EA - NOP (Implied)
    MODE_UNKNOWN,              // $EB -
    MODE_ABSOLUTE,             // $EC - CPX (Absolute)
    MODE_ABSOLUTE,             // $ED - SBC (Absolute)
    MODE_ABSOLUTE,             // $EE - INC (Absolute)
    MODE_UNKNOWN,              // $EF -
    MODE_RELATIVE,             // $F0 - BEQ (Relative)
    MODE_INDIRECT_Y,           // $F1 - SBC (Indirect,Y)
    MODE_UNKNOWN,              // $F2 -
    MODE_UNKNOWN,              // $F3 -
    MODE_UNKNOWN,              // $F4 -
    MODE_ZERO_PAGE_X,          // $F5 - SBC (Zero Page,X)
    MODE_ZERO_PAGE_X,          // $F6 - INC (Zero Page,X)
    MODE_UNKNOWN,              // $F7 -
    MODE_IMPLIED,              // $F8 - SED (Implied)
    MODE_ABSOLUTE_Y,           // $F9 - SBC (Absolute,Y)
    MODE_UNKNOWN,              // $FA -
    MODE_UNKNOWN,              // $FB -
    MODE_UNKNOWN,              // $FC -
    MODE_ABSOLUTE_X,           // $FD - SBC (Absolute,X)
    MODE_ABSOLUTE_X,           // $FE - INC (Absolute,X)
    MODE_UNKNOWN               // $FF -
  };
 
  // ---------------------------------------------------------------------------

  public static final int[] length =
  {
    1,                         // $00 - BRK (Implied)
    2,                         // $01 - ORA (Indirect,X)
    1,                         // $02 -
    1,                         // $03 -
    1,                         // $04 -
    2,                         // $05 - ORA (Zero Page)
    2,                         // $06 - ASL (Zero Page)
    1,                         // $07 -
    1,                         // $08 - PHP (Implied)
    2,                         // $09 - ORA (Immediate)
    1,                         // $0A - ASL (Accumulator)
    1,                         // $0B -
    1,                         // $0C -
    3,                         // $0D - ORA (Absolute)
    3,                         // $0E - ASL (Absolute)
    1,                         // $0F -
    2,                         // $10 - BPL (Relative)
    2,                         // $11 - ORA (Indirect,Y)
    1,                         // $12 -
    1,                         // $13 -
    1,                         // $14 -
    2,                         // $15 - ORA (Zero Page,X)
    2,                         // $16 - ASL (Zero Page,X)
    1,                         // $17 -
    1,                         // $18 - CLC (Implied)
    3,                         // $19 - ORA (Absolute,Y)
    1,                         // $1A -
    1,                         // $1B -
    1,                         // $1C -
    3,                         // $1D - ORA (Absolute,X)
    3,                         // $1E - ASL (Absolute,X)
    1,                         // $1F -
    3,                         // $20 - JSR (Absolute)
    2,                         // $21 - AND (Indirect,X)
    1,                         // $22 -
    1,                         // $23 -
    2,                         // $24 - BIT (Zero Page)
    2,                         // $25 - AND (Zero Page)
    2,                         // $26 - ROL (Zero Page)
    1,                         // $27 -
    1,                         // $28 - PLP (Implied)
    2,                         // $29 - AND (Immediate)
    1,                         // $2A - ROL (Accumulator)
    1,                         // $2B -
    3,                         // $2C - BIT (Absolute)
    3,                         // $2D - AND (Absolute)
    3,                         // $2E - ROL (Absolute)
    1,                         // $2F -
    2,                         // $30 - BMI (Relative)
    2,                         // $31 - AND (Indirect,Y)
    1,                         // $32 -
    1,                         // $33 -
    1,                         // $34 -
    2,                         // $35 - AND (Zero Page,X)
    2,                         // $36 - ROL (Zero Page,X)
    1,                         // $37 -
    1,                         // $38 - SEC (Implied)
    3,                         // $39 - AND (Absolute,Y)
    1,                         // $3A -
    1,                         // $3B -
    1,                         // $3C -
    3,                         // $3D - AND (Absolute,X)
    3,                         // $3E - ROL (Absolute,X)
    1,                         // $3F -
    1,                         // $40 - RTI (Implied)
    2,                         // $41 - EOR (Indirect,X)
    1,                         // $42 -
    1,                         // $43 -
    1,                         // $44 -
    2,                         // $45 - EOR (Zero Page)
    2,                         // $46 - LSR (Zero Page)
    1,                         // $47 -
    1,                         // $48 - PHA (Implied)
    2,                         // $49 - EOR (Immediate)
    1,                         // $4A - LSR (Accumulator)
    1,                         // $4B -
    3,                         // $4C - JMP (Absolute)
    3,                         // $4D - EOR (Absolute)
    3,                         // $4E - LSR (Absolute)
    1,                         // $4F -
    2,                         // $50 - BVC (Relative)
    2,                         // $51 - EOR (Indirect,Y)
    1,                         // $52 -
    1,                         // $53 -
    1,                         // $54 -
    2,                         // $55 - EOR (Zero Page,X)
    2,                         // $56 - LSR (Zero Page,X)
    1,                         // $57 -
    1,                         // $58 - CLI (Implied)
    3,                         // $59 - EOR (Absolute,Y)
    1,                         // $5A -
    1,                         // $5B -
    1,                         // $5C -
    3,                         // $5D - EOR (Absolute,X)
    3,                         // $5E - LSR (Absolute,X)
    1,                         // $5F -
    1,                         // $60 - RTS (Implied)
    2,                         // $61 - ADC (Indirect,X)
    1,                         // $62 -
    1,                         // $63 -
    1,                         // $64 -
    2,                         // $65 - ADC (Zero Page)
    2,                         // $66 - ROR (Zero Page)
    1,                         // $67 -
    1,                         // $68 - PLA (Implied)
    2,                         // $69 - ADC (Immediate)
    1,                         // $6A - ROR (Accumulator)
    1,                         // $6B -
    3,                         // $6C - JMP (Indirect)
    3,                         // $6D - ADC (Absolute)
    3,                         // $6E - ROR (Absolute)
    1,                         // $6F -
    2,                         // $70 - BVS (Relative)
    2,                         // $71 - ADC (Indirect,Y)
    1,                         // $72 - 
    1,                         // $73 -
    1,                         // $74 -
    2,                         // $75 - ADC (Zero Page,X)
    2,                         // $76 - ROR (Zero Page,X)
    1,                         // $77 -
    1,                         // $78 - SEI (Implied)
    3,                         // $79 - ADC (Absolute,Y)
    1,                         // $7A -
    1,                         // $7B -
    3,                         // $7C - JMP (Absolute,X)
    3,                         // $7D - ADC (Absolute,X)
    3,                         // $7E - ROR (Absolute,X)
    1,                         // $7F -
    1,                         // $80 -
    2,                         // $81 - STA (Indirect,X)
    1,                         // $82 -
    1,                         // $83 -
    2,                         // $84 - STY (Zero Page)
    2,                         // $85 - STA (Zero Page)
    2,                         // $86 - STX (Zero Page)
    1,                         // $87 -
    1,                         // $88 - DEY (Implied)
    1,                         // $89 -
    1,                         // $8A - TXA (Implied)
    1,                         // $8B -
    3,                         // $8C - STY (Absolute)
    3,                         // $8D - STA (Absolute)
    3,                         // $8E - STX (Absolute)
    1,                         // $8F -
    2,                         // $90 - BCC (Relative)
    2,                         // $91 - STA (Indirect,Y)
    1,                         // $92 -
    1,                         // $93 -
    2,                         // $94 - STY (Zero Page,X)
    2,                         // $95 - STA (Zero Page,X)
    2,                         // $96 - STX (Zero Page,Y)
    1,                         // $97 -
    1,                         // $98 - TYA (Implied)
    3,                         // $99 - STA (Absolute,Y)
    1,                         // $9A - TXS (Implied)
    1,                         // $9B -
    1,                         // $9C -
    3,                         // $9D - STA (Absolute,X)
    1,                         // $9E -
    1,                         // $9F -
    2,                         // $A0 - LDY (Immediate)
    2,                         // $A1 - LDA (Indirect,X)
    2,                         // $A2 - LDX (Immediate)
    1,                         // $A3 -
    2,                         // $A4 - LDY (Zero Page)
    2,                         // $A5 - LDA (Zero Page)
    2,                         // $A6 - LDX (Zero Page)
    1,                         // $A7 -
    1,                         // $A8 - TAY (Implied)
    2,                         // $A9 - LDA (Immediate)
    1,                         // $AA - TAX (Implied)
    1,                         // $AB -
    3,                         // $AC - LDY (Absolute)
    3,                         // $AD - LDA (Absolute)
    3,                         // $AE - LDX (Absolute)
    1,                         // $AF -
    2,                         // $B0 - BCS (Relative)
    2,                         // $B1 - LDA (Indirect,Y)
    1,                         // $B2 -
    1,                         // $B3 -
    2,                         // $B4 - LDY (Zero Page,X)
    2,                         // $B5 - LDA (Zero Page,X)
    2,                         // $B6 - LDX (Zero Page,Y)
    1,                         // $B7 -
    1,                         // $B8 - CLV (Implied)
    3,                         // $B9 - LDA (Absolute,Y)
    1,                         // $BA - TSX (Implied)
    1,                         // $BB -
    3,                         // $BC - LDY (Absolute,X)
    3,                         // $BD - LDA (Absolute,X)
    3,                         // $BE - LDX (Absolute,Y)
    1,                         // $BF -
    2,                         // $C0 - CPY (Immediate)
    2,                         // $C1 - CMP (Indirect,X)
    1,                         // $C2 -
    1,                         // $C3 -
    2,                         // $C4 - CPY (Zero Page)
    2,                         // $C5 - CMP (Zero Page)
    2,                         // $C6 - DEC (Zero Page)
    1,                         // $C7 -
    1,                         // $C8 - INY (Implied)
    2,                         // $C9 - CMP (Immediate)
    1,                         // $CA - DEX (Implied)
    1,                         // $CB -
    3,                         // $CC - CPY (Absolute)
    3,                         // $CD - CMP (Absolute)
    3,                         // $CE - DEC (Absolute)
    1,                         // $CF -
    2,                         // $D0 - BNE (Relative)
    2,                         // $D1 - CMP (Indirect,Y)
    1,                         // $D2 -
    1,                         // $D3 -
    1,                         // $D4 -
    2,                         // $D5 - CMP (Zero Page,X)
    2,                         // $D6 - DEC (Zero Page,X)
    1,                         // $D7 -
    1,                         // $D8 - CLD (Implied)
    3,                         // $D9 - CMP (Absolute,Y)
    1,                         // $DA -
    1,                         // $DB -
    1,                         // $DC -
    3,                         // $DD - CMP (Absolute,X)
    3,                         // $DE - DEC (Absolute,X)
    1,                         // $DF -
    2,                         // $E0 - CPX (Immediate)
    2,                         // $E1 - SBC (Indirect,X)
    1,                         // $E2 -
    1,                         // $E3 -
    2,                         // $E4 - CPX (Zero Page)
    2,                         // $E5 - SBC (Zero Page)
    2,                         // $E6 - INC (Zero Page)
    1,                         // $E7 -
    1,                         // $E8 - INX (Implied)
    2,                         // $E9 - SBC (Immediate)
    1,                         // $EA - NOP (Implied)
    1,                         // $EB -
    3,                         // $EC - CPX (Absolute)
    3,                         // $ED - SBC (Absolute)
    3,                         // $EE - INC (Absolute)
    1,                         // $EF -
    2,                         // $F0 - BEQ (Relative)
    2,                         // $F1 - SBC (Indirect,Y)
    1,                         // $F2 -
    1,                         // $F3 -
    1,                         // $F4 -
    2,                         // $F5 - SBC (Zero Page,X)
    2,                         // $F6 - INC (Zero Page,X)
    1,                         // $F7 -
    1,                         // $F8 - SED (Implied)
    3,                         // $F9 - SBC (Absolute,Y)
    1,                         // $FA -
    1,                         // $FB -
    1,                         // $FC -
    3,                         // $FD - SBC (Absolute,X)
    3,                         // $FE - INC (Absolute,X)
    1                          // $FF -
  };
  
  // ---------------------------------------------------------------------------
  
  // This is the BASE cycle count for each instruction
  // and does not take into account page boundry crossing.
  public static final int[] cycles =
  {
    7,                         // $00 - BRK (Implied)
    6,                         // $01 - ORA (Indirect,X)
    0,                         // $02 -
    0,                         // $03 -
    0,                         // $04 -
    2,                         // $05 - ORA (Zero Page)
    5,                         // $06 - ASL (Zero Page)
    0,                         // $07 -
    3,                         // $08 - PHP (Implied)
    2,                         // $09 - ORA (Immediate)
    2,                         // $0A - ASL (Accumulator)
    0,                         // $0B -
    0,                         // $0C -
    4,                         // $0D - ORA (Absolute)
    6,                         // $0E - ASL (Absolute)
    0,                         // $0F -
    2,                         // $10 - BPL (Relative)
    5,                         // $11 - ORA (Indirect,Y)
    0,                         // $12 -
    0,                         // $13 -
    0,                         // $14 -
    3,                         // $15 - ORA (Zero Page,X)
    6,                         // $16 - ASL (Zero Page,X)
    0,                         // $17 -
    2,                         // $18 - CLC (Implied)
    4,                         // $19 - ORA (Absolute,Y)
    0,                         // $1A -
    0,                         // $1B -
    0,                         // $1C -
    4,                         // $1D - ORA (Absolute,X)
    7,                         // $1E - ASL (Absolute,X)
    0,                         // $1F -
    6,                         // $20 - JSR (Absolute)
    6,                         // $21 - AND (Indirect,X)
    0,                         // $22 -
    0,                         // $23 -
    3,                         // $24 - BIT (Zero Page)
    2,                         // $25 - AND (Zero Page)
    5,                         // $26 - ROL (Zero Page)
    0,                         // $27 -
    4,                         // $28 - PLP (Implied)
    2,                         // $29 - AND (Immediate)
    2,                         // $2A - ROL (Accumulator)
    0,                         // $2B -
    4,                         // $2C - BIT (Absolute)
    4,                         // $2D - AND (Absolute)
    6,                         // $2E - ROL (Absolute)
    0,                         // $2F -
    2,                         // $30 - BMI (Relative)
    5,                         // $31 - AND (Indirect,Y)
    0,                         // $32 -
    0,                         // $33 -
    0,                         // $34 -
    3,                         // $35 - AND (Zero Page,X)
    6,                         // $36 - ROL (Zero Page,X)
    0,                         // $37 -
    2,                         // $38 - SEC (Implied)
    4,                         // $39 - AND (Absolute,Y)
    0,                         // $3A -
    0,                         // $3B -
    0,                         // $3C -
    4,                         // $3D - AND (Absolute,X)
    7,                         // $3E - ROL (Absolute,X)
    0,                         // $3F -
    6,                         // $40 - RTI (Implied)
    6,                         // $41 - EOR (Indirect,X)
    0,                         // $42 -
    0,                         // $43 -
    0,                         // $44 -
    3,                         // $45 - EOR (Zero Page)
    5,                         // $46 - LSR (Zero Page)
    0,                         // $47 -
    3,                         // $48 - PHA (Implied)
    2,                         // $49 - EOR (Immediate)
    2,                         // $4A - LSR (Accumulator)
    0,                         // $4B -
    3,                         // $4C - JMP (Absolute)
    4,                         // $4D - EOR (Absolute)
    6,                         // $4E - LSR (Absolute)
    0,                         // $4F -
    2,                         // $50 - BVC (Relative)
    5,                         // $51 - EOR (Indirect,Y)
    0,                         // $52 -
    0,                         // $53 -
    0,                         // $54 -
    4,                         // $55 - EOR (Zero Page,X)
    6,                         // $56 - LSR (Zero Page,X)
    0,                         // $57 -
    2,                         // $58 - CLI (Implied)
    4,                         // $59 - EOR (Absolute,Y)
    0,                         // $5A -
    0,                         // $5B -
    0,                         // $5C -
    4,                         // $5D - EOR (Absolute,X)
    7,                         // $5E - LSR (Absolute,X)
    0,                         // $5F -
    6,                         // $60 - RTS (Implied)
    6,                         // $61 - ADC (Indirect,X)
    0,                         // $62 -
    0,                         // $63 -
    0,                         // $64 -
    3,                         // $65 - ADC (Zero Page)
    5,                         // $66 - ROR (Zero Page)
    0,                         // $67 -
    4,                         // $68 - PLA (Implied)
    2,                         // $69 - ADC (Immediate)
    2,                         // $6A - ROR (Accumulator)
    0,                         // $6B -
    5,                         // $6C - JMP (Indirect)
    4,                         // $6D - ADC (Absolute)
    6,                         // $6E - ROR (Absolute)
    0,                         // $6F -
    2,                         // $70 - BVS (Relative)
    5,                         // $71 - ADC (Indirect,Y)
    0,                         // $72 - 
    0,                         // $73 -
    0,                         // $74 -
    4,                         // $75 - ADC (Zero Page,X)
    6,                         // $76 - ROR (Zero Page,X)
    0,                         // $77 -
    2,                         // $78 - SEI (Implied)
    4,                         // $79 - ADC (Absolute,Y)
    0,                         // $7A -
    0,                         // $7B -
    6,                         // $7C - JMP (Absolute,X)
    4,                         // $7D - ADC (Absolute,X)
    7,                         // $7E - ROR (Absolute,X)
    0,                         // $7F -
    0,                         // $80 -
    6,                         // $81 - STA (Indirect,X)
    0,                         // $82 -
    0,                         // $83 -
    3,                         // $84 - STY (Zero Page)
    3,                         // $85 - STA (Zero Page)
    3,                         // $86 - STX (Zero Page)
    0,                         // $87 -
    2,                         // $88 - DEY (Implied)
    0,                         // $89 -
    2,                         // $8A - TXA (Implied)
    0,                         // $8B -
    4,                         // $8C - STY (Absolute)
    4,                         // $8D - STA (Absolute)
    4,                         // $8E - STX (Absolute)
    0,                         // $8F -
    2,                         // $90 - BCC (Relative)
    6,                         // $91 - STA (Indirect,Y)
    0,                         // $92 -
    0,                         // $93 -
    4,                         // $94 - STY (Zero Page,X)
    4,                         // $95 - STA (Zero Page,X)
    4,                         // $96 - STX (Zero Page,Y)
    0,                         // $97 -
    2,                         // $98 - TYA (Implied)
    5,                         // $99 - STA (Absolute,Y)
    2,                         // $9A - TXS (Implied)
    0,                         // $9B -
    0,                         // $9C -
    5,                         // $9D - STA (Absolute,X)
    0,                         // $9E -
    0,                         // $9F -
    2,                         // $A0 - LDY (Immediate)
    6,                         // $A1 - LDA (Indirect,X)
    2,                         // $A2 - LDX (Immediate)
    0,                         // $A3 -
    3,                         // $A4 - LDY (Zero Page)
    3,                         // $A5 - LDA (Zero Page)
    3,                         // $A6 - LDX (Zero Page)
    0,                         // $A7 -
    2,                         // $A8 - TAY (Implied)
    2,                         // $A9 - LDA (Immediate)
    2,                         // $AA - TAX (Implied)
    0,                         // $AB -
    4,                         // $AC - LDY (Absolute)
    4,                         // $AD - LDA (Absolute)
    4,                         // $AE - LDX (Absolute)
    0,                         // $AF -
    2,                         // $B0 - BCS (Relative)
    5,                         // $B1 - LDA (Indirect,Y)
    0,                         // $B2 -
    0,                         // $B3 -
    4,                         // $B4 - LDY (Zero Page,X)
    4,                         // $B5 - LDA (Zero Page,X)
    4,                         // $B6 - LDX (Zero Page,Y)
    0,                         // $B7 -
    2,                         // $B8 - CLV (Implied)
    4,                         // $B9 - LDA (Absolute,Y)
    2,                         // $BA - TSX (Implied)
    0,                         // $BB -
    4,                         // $BC - LDY (Absolute,X)
    4,                         // $BD - LDA (Absolute,X)
    4,                         // $BE - LDX (Absolute,Y)
    0,                         // $BF -
    2,                         // $C0 - CPY (Immediate)
    6,                         // $C1 - CMP (Indirect,X)
    0,                         // $C2 -
    0,                         // $C3 -
    3,                         // $C4 - CPY (Zero Page)
    3,                         // $C5 - CMP (Zero Page)
    5,                         // $C6 - DEC (Zero Page)
    0,                         // $C7 -
    2,                         // $C8 - INY (Implied)
    2,                         // $C9 - CMP (Immediate)
    2,                         // $CA - DEX (Implied)
    0,                         // $CB -
    4,                         // $CC - CPY (Absolute)
    4,                         // $CD - CMP (Absolute)
    6,                         // $CE - DEC (Absolute)
    0,                         // $CF -
    2,                         // $D0 - BNE (Relative)
    5,                         // $D1 - CMP (Indirect,Y)
    0,                         // $D2 -
    0,                         // $D3 -
    0,                         // $D4 -
    4,                         // $D5 - CMP (Zero Page,X)
    6,                         // $D6 - DEC (Zero Page,X)
    0,                         // $D7 -
    2,                         // $D8 - CLD (Implied)
    4,                         // $D9 - CMP (Absolute,Y)
    0,                         // $DA -
    0,                         // $DB -
    0,                         // $DC -
    4,                         // $DD - CMP (Absolute,X)
    7,                         // $DE - DEC (Absolute,X)
    0,                         // $DF -
    2,                         // $E0 - CPX (Immediate)
    6,                         // $E1 - SBC (Indirect,X)
    0,                         // $E2 -
    0,                         // $E3 -
    3,                         // $E4 - CPX (Zero Page)
    3,                         // $E5 - SBC (Zero Page)
    5,                         // $E6 - INC (Zero Page)
    0,                         // $E7 -
    2,                         // $E8 - INX (Implied)
    2,                         // $E9 - SBC (Immediate)
    2,                         // $EA - NOP (Implied)
    0,                         // $EB -
    4,                         // $EC - CPX (Absolute)
    4,                         // $ED - SBC (Absolute)
    6,                         // $EE - INC (Absolute)
    0,                         // $EF -
    2,                         // $F0 - BEQ (Relative)
    5,                         // $F1 - SBC (Indirect,Y)
    0,                         // $F2 -
    0,                         // $F3 -
    0,                         // $F4 -
    4,                         // $F5 - SBC (Zero Page,X)
    6,                         // $F6 - INC (Zero Page,X)
    0,                         // $F7 -
    2,                         // $F8 - SED (Implied)
    4,                         // $F9 - SBC (Absolute,Y)
    0,                         // $FA -
    0,                         // $FB -
    0,                         // $FC -
    4,                         // $FD - SBC (Absolute,X)
    7,                         // $FE - INC (Absolute,X)
    0                          // $FF -
  };
  
  // ---------------------------------------------------------------------------
  
  // This is the Page Boundry Crossing Extra Cycle Count.
  public static final int[] pb_cycles =
  {
    0,                         // $00 - BRK (Implied)
    0,                         // $01 - ORA (Indirect,X)
    0,                         // $02 -
    0,                         // $03 -
    0,                         // $04 -
    0,                         // $05 - ORA (Zero Page)
    0,                         // $06 - ASL (Zero Page)
    0,                         // $07 -
    0,                         // $08 - PHP (Implied)
    0,                         // $09 - ORA (Immediate)
    0,                         // $0A - ASL (Accumulator)
    0,                         // $0B -
    0,                         // $0C -
    0,                         // $0D - ORA (Absolute)
    0,                         // $0E - ASL (Absolute)
    0,                         // $0F -
    1,                         // $10 - BPL (Relative)
    1,                         // $11 - ORA (Indirect,Y)
    0,                         // $12 -
    0,                         // $13 -
    0,                         // $14 -
    0,                         // $15 - ORA (Zero Page,X)
    0,                         // $16 - ASL (Zero Page,X)
    0,                         // $17 -
    0,                         // $18 - CLC (Implied)
    1,                         // $19 - ORA (Absolute,Y)
    0,                         // $1A -
    0,                         // $1B -
    0,                         // $1C -
    1,                         // $1D - ORA (Absolute,X)
    0,                         // $1E - ASL (Absolute,X)
    0,                         // $1F -
    0,                         // $20 - JSR (Absolute)
    0,                         // $21 - AND (Indirect,X)
    0,                         // $22 -
    0,                         // $23 -
    0,                         // $24 - BIT (Zero Page)
    0,                         // $25 - AND (Zero Page)
    0,                         // $26 - ROL (Zero Page)
    0,                         // $27 -
    0,                         // $28 - PLP (Implied)
    0,                         // $29 - AND (Immediate)
    0,                         // $2A - ROL (Accumulator)
    0,                         // $2B -
    0,                         // $2C - BIT (Absolute)
    0,                         // $2D - AND (Absolute)
    0,                         // $2E - ROL (Absolute)
    0,                         // $2F -
    1,                         // $30 - BMI (Relative)
    1,                         // $31 - AND (Indirect,Y)
    0,                         // $32 -
    0,                         // $33 -
    0,                         // $34 -
    0,                         // $35 - AND (Zero Page,X)
    0,                         // $36 - ROL (Zero Page,X)
    0,                         // $37 -
    0,                         // $38 - SEC (Implied)
    1,                         // $39 - AND (Absolute,Y)
    0,                         // $3A -
    0,                         // $3B -
    0,                         // $3C -
    1,                         // $3D - AND (Absolute,X)
    0,                         // $3E - ROL (Absolute,X)
    0,                         // $3F -
    0,                         // $40 - RTI (Implied)
    0,                         // $41 - EOR (Indirect,X)
    0,                         // $42 -
    0,                         // $43 -
    0,                         // $44 -
    0,                         // $45 - EOR (Zero Page)
    0,                         // $46 - LSR (Zero Page)
    0,                         // $47 -
    0,                         // $48 - PHA (Implied)
    0,                         // $49 - EOR (Immediate)
    0,                         // $4A - LSR (Accumulator)
    0,                         // $4B -
    0,                         // $4C - JMP (Absolute)
    0,                         // $4D - EOR (Absolute)
    0,                         // $4E - LSR (Absolute)
    0,                         // $4F -
    1,                         // $50 - BVC (Relative)
    1,                         // $51 - EOR (Indirect,Y)
    0,                         // $52 -
    0,                         // $53 -
    0,                         // $54 -
    0,                         // $55 - EOR (Zero Page,X)
    0,                         // $56 - LSR (Zero Page,X)
    0,                         // $57 -
    0,                         // $58 - CLI (Implied)
    1,                         // $59 - EOR (Absolute,Y)
    0,                         // $5A -
    0,                         // $5B -
    0,                         // $5C -
    1,                         // $5D - EOR (Absolute,X)
    0,                         // $5E - LSR (Absolute,X)
    0,                         // $5F -
    0,                         // $60 - RTS (Implied)
    0,                         // $61 - ADC (Indirect,X)
    0,                         // $62 -
    0,                         // $63 -
    0,                         // $64 -
    0,                         // $65 - ADC (Zero Page)
    0,                         // $66 - ROR (Zero Page)
    0,                         // $67 -
    0,                         // $68 - PLA (Implied)
    0,                         // $69 - ADC (Immediate)
    0,                         // $6A - ROR (Accumulator)
    0,                         // $6B -
    0,                         // $6C - JMP (Indirect)
    0,                         // $6D - ADC (Absolute)
    0,                         // $6E - ROR (Absolute)
    0,                         // $6F -
    1,                         // $70 - BVS (Relative)
    1,                         // $71 - ADC (Indirect,Y)
    0,                         // $72 - 
    0,                         // $73 -
    0,                         // $74 -
    0,                         // $75 - ADC (Zero Page,X)
    0,                         // $76 - ROR (Zero Page,X)
    0,                         // $77 -
    0,                         // $78 - SEI (Implied)
    1,                         // $79 - ADC (Absolute,Y)
    0,                         // $7A -
    0,                         // $7B -
    0,                         // $7C - JMP (Absolute,X)
    1,                         // $7D - ADC (Absolute,X)
    0,                         // $7E - ROR (Absolute,X)
    0,                         // $7F -
    0,                         // $80 -
    0,                         // $81 - STA (Indirect,X)
    0,                         // $82 -
    0,                         // $83 -
    0,                         // $84 - STY (Zero Page)
    0,                         // $85 - STA (Zero Page)
    0,                         // $86 - STX (Zero Page)
    0,                         // $87 -
    0,                         // $88 - DEY (Implied)
    0,                         // $89 -
    0,                         // $8A - TXA (Implied)
    0,                         // $8B -
    0,                         // $8C - STY (Absolute)
    0,                         // $8D - STA (Absolute)
    0,                         // $8E - STX (Absolute)
    0,                         // $8F -
    1,                         // $90 - BCC (Relative)
    0,                         // $91 - STA (Indirect,Y)
    0,                         // $92 -
    0,                         // $93 -
    0,                         // $94 - STY (Zero Page,X)
    0,                         // $95 - STA (Zero Page,X)
    0,                         // $96 - STX (Zero Page,Y)
    0,                         // $97 -
    0,                         // $98 - TYA (Implied)
    0,                         // $99 - STA (Absolute,Y)
    0,                         // $9A - TXS (Implied)
    0,                         // $9B -
    0,                         // $9C -
    0,                         // $9D - STA (Absolute,X)
    0,                         // $9E -
    0,                         // $9F -
    0,                         // $A0 - LDY (Immediate)
    0,                         // $A1 - LDA (Indirect,X)
    0,                         // $A2 - LDX (Immediate)
    0,                         // $A3 -
    0,                         // $A4 - LDY (Zero Page)
    0,                         // $A5 - LDA (Zero Page)
    0,                         // $A6 - LDX (Zero Page)
    0,                         // $A7 -
    0,                         // $A8 - TAY (Implied)
    0,                         // $A9 - LDA (Immediate)
    0,                         // $AA - TAX (Implied)
    0,                         // $AB -
    0,                         // $AC - LDY (Absolute)
    0,                         // $AD - LDA (Absolute)
    0,                         // $AE - LDX (Absolute)
    0,                         // $AF -
    1,                         // $B0 - BCS (Relative)
    1,                         // $B1 - LDA (Indirect,Y)
    0,                         // $B2 -
    0,                         // $B3 -
    0,                         // $B4 - LDY (Zero Page,X)
    0,                         // $B5 - LDA (Zero Page,X)
    0,                         // $B6 - LDX (Zero Page,Y)
    0,                         // $B7 -
    0,                         // $B8 - CLV (Implied)
    1,                         // $B9 - LDA (Absolute,Y)
    0,                         // $BA - TSX (Implied)
    0,                         // $BB -
    1,                         // $BC - LDY (Absolute,X)
    1,                         // $BD - LDA (Absolute,X)
    1,                         // $BE - LDX (Absolute,Y)
    0,                         // $BF -
    0,                         // $C0 - CPY (Immediate)
    0,                         // $C1 - CMP (Indirect,X)
    0,                         // $C2 -
    0,                         // $C3 -
    0,                         // $C4 - CPY (Zero Page)
    0,                         // $C5 - CMP (Zero Page)
    0,                         // $C6 - DEC (Zero Page)
    0,                         // $C7 -
    0,                         // $C8 - INY (Implied)
    0,                         // $C9 - CMP (Immediate)
    0,                         // $CA - DEX (Implied)
    0,                         // $CB -
    0,                         // $CC - CPY (Absolute)
    0,                         // $CD - CMP (Absolute)
    0,                         // $CE - DEC (Absolute)
    0,                         // $CF -
    1,                         // $D0 - BNE (Relative)
    1,                         // $D1 - CMP (Indirect,Y)
    0,                         // $D2 -
    0,                         // $D3 -
    0,                         // $D4 -
    0,                         // $D5 - CMP (Zero Page,X)
    0,                         // $D6 - DEC (Zero Page,X)
    0,                         // $D7 -
    0,                         // $D8 - CLD (Implied)
    1,                         // $D9 - CMP (Absolute,Y)
    0,                         // $DA -
    0,                         // $DB -
    0,                         // $DC -
    1,                         // $DD - CMP (Absolute,X)
    0,                         // $DE - DEC (Absolute,X)
    0,                         // $DF -
    0,                         // $E0 - CPX (Immediate)
    0,                         // $E1 - SBC (Indirect,X)
    0,                         // $E2 -
    0,                         // $E3 -
    0,                         // $E4 - CPX (Zero Page)
    0,                         // $E5 - SBC (Zero Page)
    0,                         // $E6 - INC (Zero Page)
    0,                         // $E7 -
    0,                         // $E8 - INX (Implied)
    0,                         // $E9 - SBC (Immediate)
    0,                         // $EA - NOP (Implied)
    0,                         // $EB -
    0,                         // $EC - CPX (Absolute)
    0,                         // $ED - SBC (Absolute)
    0,                         // $EE - INC (Absolute)
    0,                         // $EF -
    1,                         // $F0 - BEQ (Relative)
    1,                         // $F1 - SBC (Indirect,Y)
    0,                         // $F2 -
    0,                         // $F3 -
    0,                         // $F4 -
    0,                         // $F5 - SBC (Zero Page,X)
    0,                         // $F6 - INC (Zero Page,X)
    0,                         // $F7 -
    0,                         // $F8 - SED (Implied)
    1,                         // $F9 - SBC (Absolute,Y)
    0,                         // $FA -
    0,                         // $FB -
    0,                         // $FC -
    1,                         // $FD - SBC (Absolute,X)
    0,                         // $FE - INC (Absolute,X)
    0                          // $FF -
  };
  
  // ---------------------------------------------------------------------------
  
  // This is the Branch Taken extra cycles count.
  public static final int[] bt_cycles =
  {
    0,                         // $00 - BRK (Implied)
    0,                         // $01 - ORA (Indirect,X)
    0,                         // $02 -
    0,                         // $03 -
    0,                         // $04 -
    0,                         // $05 - ORA (Zero Page)
    0,                         // $06 - ASL (Zero Page)
    0,                         // $07 -
    0,                         // $08 - PHP (Implied)
    0,                         // $09 - ORA (Immediate)
    0,                         // $0A - ASL (Accumulator)
    0,                         // $0B -
    0,                         // $0C -
    0,                         // $0D - ORA (Absolute)
    0,                         // $0E - ASL (Absolute)
    0,                         // $0F -
    1,                         // $10 - BPL (Relative)
    0,                         // $11 - ORA (Indirect,Y)
    0,                         // $12 -
    0,                         // $13 -
    0,                         // $14 -
    0,                         // $15 - ORA (Zero Page,X)
    0,                         // $16 - ASL (Zero Page,X)
    0,                         // $17 -
    0,                         // $18 - CLC (Implied)
    0,                         // $19 - ORA (Absolute,Y)
    0,                         // $1A -
    0,                         // $1B -
    0,                         // $1C -
    0,                         // $1D - ORA (Absolute,X)
    0,                         // $1E - ASL (Absolute,X)
    0,                         // $1F -
    0,                         // $20 - JSR (Absolute)
    0,                         // $21 - AND (Indirect,X)
    0,                         // $22 -
    0,                         // $23 -
    0,                         // $24 - BIT (Zero Page)
    0,                         // $25 - AND (Zero Page)
    0,                         // $26 - ROL (Zero Page)
    0,                         // $27 -
    0,                         // $28 - PLP (Implied)
    0,                         // $29 - AND (Immediate)
    0,                         // $2A - ROL (Accumulator)
    0,                         // $2B -
    0,                         // $2C - BIT (Absolute)
    0,                         // $2D - AND (Absolute)
    0,                         // $2E - ROL (Absolute)
    0,                         // $2F -
    1,                         // $30 - BMI (Relative)
    0,                         // $31 - AND (Indirect,Y)
    0,                         // $32 -
    0,                         // $33 -
    0,                         // $34 -
    0,                         // $35 - AND (Zero Page,X)
    0,                         // $36 - ROL (Zero Page,X)
    0,                         // $37 -
    0,                         // $38 - SEC (Implied)
    0,                         // $39 - AND (Absolute,Y)
    0,                         // $3A -
    0,                         // $3B -
    0,                         // $3C -
    0,                         // $3D - AND (Absolute,X)
    0,                         // $3E - ROL (Absolute,X)
    0,                         // $3F -
    0,                         // $40 - RTI (Implied)
    0,                         // $41 - EOR (Indirect,X)
    0,                         // $42 -
    0,                         // $43 -
    0,                         // $44 -
    0,                         // $45 - EOR (Zero Page)
    0,                         // $46 - LSR (Zero Page)
    0,                         // $47 -
    0,                         // $48 - PHA (Implied)
    0,                         // $49 - EOR (Immediate)
    0,                         // $4A - LSR (Accumulator)
    0,                         // $4B -
    0,                         // $4C - JMP (Absolute)
    0,                         // $4D - EOR (Absolute)
    0,                         // $4E - LSR (Absolute)
    0,                         // $4F -
    1,                         // $50 - BVC (Relative)
    0,                         // $51 - EOR (Indirect,Y)
    0,                         // $52 -
    0,                         // $53 -
    0,                         // $54 -
    0,                         // $55 - EOR (Zero Page,X)
    0,                         // $56 - LSR (Zero Page,X)
    0,                         // $57 -
    0,                         // $58 - CLI (Implied)
    0,                         // $59 - EOR (Absolute,Y)
    0,                         // $5A -
    0,                         // $5B -
    0,                         // $5C -
    0,                         // $5D - EOR (Absolute,X)
    0,                         // $5E - LSR (Absolute,X)
    0,                         // $5F -
    0,                         // $60 - RTS (Implied)
    0,                         // $61 - ADC (Indirect,X)
    0,                         // $62 -
    0,                         // $63 -
    0,                         // $64 -
    0,                         // $65 - ADC (Zero Page)
    0,                         // $66 - ROR (Zero Page)
    0,                         // $67 -
    0,                         // $68 - PLA (Implied)
    0,                         // $69 - ADC (Immediate)
    0,                         // $6A - ROR (Accumulator)
    0,                         // $6B -
    0,                         // $6C - JMP (Indirect)
    0,                         // $6D - ADC (Absolute)
    0,                         // $6E - ROR (Absolute)
    0,                         // $6F -
    1,                         // $70 - BVS (Relative)
    0,                         // $71 - ADC (Indirect,Y)
    0,                         // $72 - 
    0,                         // $73 -
    0,                         // $74 -
    0,                         // $75 - ADC (Zero Page,X)
    0,                         // $76 - ROR (Zero Page,X)
    0,                         // $77 -
    0,                         // $78 - SEI (Implied)
    0,                         // $79 - ADC (Absolute,Y)
    0,                         // $7A -
    0,                         // $7B -
    0,                         // $7C - JMP (Absolute,X)
    0,                         // $7D - ADC (Absolute,X)
    0,                         // $7E - ROR (Absolute,X)
    0,                         // $7F -
    0,                         // $80 -
    0,                         // $81 - STA (Indirect,X)
    0,                         // $82 -
    0,                         // $83 -
    0,                         // $84 - STY (Zero Page)
    0,                         // $85 - STA (Zero Page)
    0,                         // $86 - STX (Zero Page)
    0,                         // $87 -
    0,                         // $88 - DEY (Implied)
    0,                         // $89 -
    0,                         // $8A - TXA (Implied)
    0,                         // $8B -
    0,                         // $8C - STY (Absolute)
    0,                         // $8D - STA (Absolute)
    0,                         // $8E - STX (Absolute)
    0,                         // $8F -
    1,                         // $90 - BCC (Relative)
    0,                         // $91 - STA (Indirect,Y)
    0,                         // $92 -
    0,                         // $93 -
    0,                         // $94 - STY (Zero Page,X)
    0,                         // $95 - STA (Zero Page,X)
    0,                         // $96 - STX (Zero Page,Y)
    0,                         // $97 -
    0,                         // $98 - TYA (Implied)
    0,                         // $99 - STA (Absolute,Y)
    0,                         // $9A - TXS (Implied)
    0,                         // $9B -
    0,                         // $9C -
    0,                         // $9D - STA (Absolute,X)
    0,                         // $9E -
    0,                         // $9F -
    0,                         // $A0 - LDY (Immediate)
    0,                         // $A1 - LDA (Indirect,X)
    0,                         // $A2 - LDX (Immediate)
    0,                         // $A3 -
    0,                         // $A4 - LDY (Zero Page)
    0,                         // $A5 - LDA (Zero Page)
    0,                         // $A6 - LDX (Zero Page)
    0,                         // $A7 -
    0,                         // $A8 - TAY (Implied)
    0,                         // $A9 - LDA (Immediate)
    0,                         // $AA - TAX (Implied)
    0,                         // $AB -
    0,                         // $AC - LDY (Absolute)
    0,                         // $AD - LDA (Absolute)
    0,                         // $AE - LDX (Absolute)
    0,                         // $AF -
    1,                         // $B0 - BCS (Relative)
    0,                         // $B1 - LDA (Indirect,Y)
    0,                         // $B2 -
    0,                         // $B3 -
    0,                         // $B4 - LDY (Zero Page,X)
    0,                         // $B5 - LDA (Zero Page,X)
    0,                         // $B6 - LDX (Zero Page,Y)
    0,                         // $B7 -
    0,                         // $B8 - CLV (Implied)
    0,                         // $B9 - LDA (Absolute,Y)
    0,                         // $BA - TSX (Implied)
    0,                         // $BB -
    0,                         // $BC - LDY (Absolute,X)
    0,                         // $BD - LDA (Absolute,X)
    0,                         // $BE - LDX (Absolute,Y)
    0,                         // $BF -
    0,                         // $C0 - CPY (Immediate)
    0,                         // $C1 - CMP (Indirect,X)
    0,                         // $C2 -
    0,                         // $C3 -
    0,                         // $C4 - CPY (Zero Page)
    0,                         // $C5 - CMP (Zero Page)
    0,                         // $C6 - DEC (Zero Page)
    0,                         // $C7 -
    0,                         // $C8 - INY (Implied)
    0,                         // $C9 - CMP (Immediate)
    0,                         // $CA - DEX (Implied)
    0,                         // $CB -
    0,                         // $CC - CPY (Absolute)
    0,                         // $CD - CMP (Absolute)
    0,                         // $CE - DEC (Absolute)
    0,                         // $CF -
    1,                         // $D0 - BNE (Relative)
    0,                         // $D1 - CMP (Indirect,Y)
    0,                         // $D2 -
    0,                         // $D3 -
    0,                         // $D4 -
    0,                         // $D5 - CMP (Zero Page,X)
    0,                         // $D6 - DEC (Zero Page,X)
    0,                         // $D7 -
    0,                         // $D8 - CLD (Implied)
    0,                         // $D9 - CMP (Absolute,Y)
    0,                         // $DA -
    0,                         // $DB -
    0,                         // $DC -
    0,                         // $DD - CMP (Absolute,X)
    0,                         // $DE - DEC (Absolute,X)
    0,                         // $DF -
    0,                         // $E0 - CPX (Immediate)
    0,                         // $E1 - SBC (Indirect,X)
    0,                         // $E2 -
    0,                         // $E3 -
    0,                         // $E4 - CPX (Zero Page)
    0,                         // $E5 - SBC (Zero Page)
    0,                         // $E6 - INC (Zero Page)
    0,                         // $E7 -
    0,                         // $E8 - INX (Implied)
    0,                         // $E9 - SBC (Immediate)
    0,                         // $EA - NOP (Implied)
    0,                         // $EB -
    0,                         // $EC - CPX (Absolute)
    0,                         // $ED - SBC (Absolute)
    0,                         // $EE - INC (Absolute)
    0,                         // $EF -
    1,                         // $F0 - BEQ (Relative)
    0,                         // $F1 - SBC (Indirect,Y)
    0,                         // $F2 -
    0,                         // $F3 -
    0,                         // $F4 -
    0,                         // $F5 - SBC (Zero Page,X)
    0,                         // $F6 - INC (Zero Page,X)
    0,                         // $F7 -
    0,                         // $F8 - SED (Implied)
    0,                         // $F9 - SBC (Absolute,Y)
    0,                         // $FA -
    0,                         // $FB -
    0,                         // $FC -
    0,                         // $FD - SBC (Absolute,X)
    0,                         // $FE - INC (Absolute,X)
    0                          // $FF -
  };
  
  // ---------------------------------------------------------------------------  
}