package com.monkeycoder.monkeynes.hardware;

import com.monkeycoder.monkeynes.base.*;

import java.util.*;

public class NesVideoMemory extends Observable
{
  // 64k (4x shadowed in 16k chunks)
  private int[] memory = new int[0x00010000];
  
  public NesVideoMemory()
  {
    // Do nothing.
  }

  // -------------------------------------------------------------------  
  public int read( int addr )
  {
    setChanged();
    notifyObservers( new MemoryAccessMessage( MemoryAccessMessage.MEM_VIDEO, MemoryAccessMessage.TYPE_READ, addr ) );
    
    return memory[MathHelper.getWrappedAddress(addr)];
  }
  
  public int readNoMessage( int addr )
  {
    return memory[MathHelper.getWrappedAddress(addr)];
  }  

  public int readWord( int addr )
  {
    setChanged();
    notifyObservers( new MemoryAccessMessage( MemoryAccessMessage.MEM_VIDEO, MemoryAccessMessage.TYPE_READ_WORD, addr, addr + 1 ) );
    
    return (memory[MathHelper.getWrappedAddress(addr + 1)] << 8) | memory[MathHelper.getWrappedAddress(addr)];
  }
  
  public int readWordNoMessage( int addr )
  {
    return (memory[MathHelper.getWrappedAddress(addr + 1)] << 8) | memory[MathHelper.getWrappedAddress(addr)];
  }  

  public int[] read( int startAddr, int stopAddr )
  {
    setChanged();
    notifyObservers( new MemoryAccessMessage( MemoryAccessMessage.MEM_VIDEO, MemoryAccessMessage.TYPE_READ_RANGE, startAddr, stopAddr ) );
    
    int j = 0;
    int[] temp = new int[stopAddr - startAddr + 1];
    
    for( int i = startAddr; i <= stopAddr; i++ )
    {
      temp[j] = readNoMessage(i);
      j++;
    }
    
    return temp;
  }  
  
  public int[] readNoMessage( int startAddr, int stopAddr )
  {
    int j = 0;
    int[] temp = new int[stopAddr - startAddr + 1];
    
    for( int i = startAddr; i <= stopAddr; i++ )
    {
      temp[j] = readNoMessage(i);
      j++;
    }
    
    return temp;
  }  
  
  // -------------------------------------------------------------------
  public void write( int addr, int val )
  {
    // Shadow it
    memory[MathHelper.getWrappedAddress(addr + 0x00000000)] = val;
    memory[MathHelper.getWrappedAddress(addr + 0x00004000)] = val;
    memory[MathHelper.getWrappedAddress(addr + 0x00008000)] = val;
    memory[MathHelper.getWrappedAddress(addr + 0x0000C000)] = val;

    setChanged();
    notifyObservers( new MemoryAccessMessage( MemoryAccessMessage.MEM_VIDEO, MemoryAccessMessage.TYPE_WRITE, addr ) );
  }
  
  public void writeNoMessage( int addr, int val )
  {
    // Shadow it
    memory[MathHelper.getWrappedAddress(addr + 0x00000000)] = val;
    memory[MathHelper.getWrappedAddress(addr + 0x00004000)] = val;
    memory[MathHelper.getWrappedAddress(addr + 0x00008000)] = val;
    memory[MathHelper.getWrappedAddress(addr + 0x0000C000)] = val;
  }  
  
  public void write( int startAddr, int[] data )
  {
    // writes the whole input buffer
    for( int i = 0; i < data.length; i++ )
      writeNoMessage( startAddr + i, data[i] );
    
    setChanged();
    notifyObservers( new MemoryAccessMessage( MemoryAccessMessage.MEM_VIDEO, MemoryAccessMessage.TYPE_WRITE_RANGE, startAddr, startAddr + data.length - 1 ) );
  }
  
  public void writeNoMessage( int startAddr, int[] data )
  {
    // writes the whole input buffer
    for( int i = 0; i < data.length; i++ )
      writeNoMessage( startAddr + i, data[i] );
  }  

  public void write( int startAddr, int[] data, int len )
  {
    // writes up to len of the input buffer
    for( int i = 0; i < len; i++ )
      writeNoMessage( startAddr + i, data[i] );

    setChanged();
    notifyObservers( new MemoryAccessMessage( MemoryAccessMessage.MEM_VIDEO, MemoryAccessMessage.TYPE_WRITE_RANGE, startAddr, startAddr + len - 1 ) );
  }

  public void writeNoMessage( int startAddr, int[] data, int len )
  {
    // writes up to len of the input buffer
    for( int i = 0; i < len; i++ )
      writeNoMessage( startAddr + i, data[i] );
  }  

}